
{$DEFINE WIN2000}

unit AboutUnit;

interface

uses Windows, Messages, CommCtrl, SystemGlobal;

type
  { CDialog }
  CDialog = class
  private
    m_uId: UINT;
    m_hDlg: HWND;
    m_hParent: HWND;
  protected

  public
    constructor Create(uId: UINT; hParent: HWND);
    destructor Destroy; override;
    function DoModal: Integer;
    procedure OnCancel; virtual;
    procedure OnDestroy; virtual;
    function OnInitDialog: BOOL; virtual;
    procedure OnOK; virtual;
    procedure Show;
    function ShowModal: Integer;
    procedure ShowWindow(nShow: Integer);
    procedure WMCommand(var Msg: TMessage); virtual;
    procedure WMInitDialog(var Msg: TMessage); virtual;
    procedure WndProc(var Msg: TMessage); virtual;
    property hDlg: HWND read m_hDlg;
  end;

  { CCoolDialog }

  CCoolDialog = class(CDialog)
  private
    m_uStyle: UINT;
    m_bAlpha: Byte;
    m_crTransparent: COLORREF;
    {$IFDEF WIN2000}
    procedure SetExLayeredWindow;
    function ValidStyle: Bool;
    {$ENDIF}
  protected
    {$IFDEF WIN2000}
    procedure SetAlpha(bValue: Byte);
    procedure SetTransparent(crValue: COLORREF);
    {$ENDIF}
  public
    constructor Create(uId: UINT; hParent: HWND; uStyle: UINT; dwValue: DWORD);
    procedure OnDestroy; override;
    function OnInitDialog: BOOL; override;
    procedure WMLButtonDown(var Msg: TMessage); virtual;
    procedure WndProc(var Msg: TMessage); override;
    {$IFDEF WIN2000}
    property Alpha: Byte read m_bAlpha write SetAlpha;
    property Transparent: COLORREF read m_crTransparent write SetTransparent;
    {$ENDIF}
  end;

  { CAboutDialog }

  CAboutDialog = class(CCoolDialog)
  private
    m_hFontProduct: HFONT;
    m_hbrBack: HBRUSH;
  public
    constructor Create(hWndMain: HWND; bAlpha: Byte);
    procedure OnDestroy; override;
    function OnInitDialog: BOOL; override;
    procedure WMCtlColorDlg(var Msg: TMessage); virtual;
    procedure WndProc(var Msg: TMessage); override;
  end;

implementation

uses SysUtils;

const
  IDD_ABOUT_DLG = 105;

function DialogProc( hDlg: HWND; uMsg: UINT; wParam, lParam: LongInt ): LRESULT; stdcall;
var
  Dialog: CDialog;
  Msg: TMessage;
begin

  Result := 0;

  Msg.Msg := uMsg;
  Msg.WParam := wParam;
  Msg.LParam := lParam;
  Msg.Result := Integer( Result );

  Dialog := CDialog( GetWindowLong( hDlg, GWL_USERDATA ) );

  if not Assigned( Dialog ) then
  begin
    if uMsg = WM_INITDIALOG then
    begin
      Dialog := CDialog( lParam );
      Dialog.m_hDlg := hDlg;
      SetWindowLong( hDlg, GWL_USERDATA, LongInt( Dialog ) );
      Dialog.WndProc( Msg );
    end;
  end
  else Dialog.WndProc( Msg );

  Result := Msg.Result;

end;

{ CDialog }

constructor CDialog.Create(uId: UINT; hParent: HWND);
begin
  m_uId :=      uId;
  m_hParent :=  hParent;
  m_hDlg :=     HWND(0);
end;

destructor CDialog.Destroy;
begin
  inherited Destroy;
end;

function CDialog.DoModal: Integer;
begin
  Result := ShowModal;
end;

procedure CDialog.OnCancel;
begin
  EndDialog( m_hDlg, IDCANCEL );
end;

procedure CDialog.OnDestroy;
begin

end;

function CDialog.OnInitDialog: BOOL;
begin
  Result := False;
end;

procedure CDialog.OnOK;
begin
  EndDialog( m_hDlg, IDOK );
end;

procedure CDialog.Show;
begin
  if not IsWindow( m_hDlg ) then
  begin
    CreateDialogParam( SysInit.HInstance, MakeIntResource( m_uId ),
      m_hParent, @DialogProc, Integer( Self ) );
    ShowWindow( SW_SHOW );
  end;
end;

function CDialog.ShowModal: Integer;
var
  hWndFocus: HWND;
begin
  hWndFocus := GetFocus;
  Result := DialogBoxParam( SysInit.HInstance, MakeIntResource( m_uId ),
    m_hParent, @DialogProc, Integer( Self ) );
  SetFocus( hWndFocus );
end;

procedure CDialog.ShowWindow(nShow: Integer);
begin
  Windows.ShowWindow( m_hDlg, nShow );
end;

procedure CDialog.WMCommand(var Msg: TMessage);
begin
  case Msg.WParamLo of
    IDOK     : OnOK;
    IDCANCEL : OnCancel;
  end;
end;

procedure CDialog.WMInitDialog(var Msg: TMessage);
begin
  Msg.Result := LRESULT( OnInitDialog );
end;

procedure CDialog.WndProc(var Msg: TMessage);
begin
  case Msg.Msg of
    WM_COMMAND:     WMCommand( Msg );
    WM_INITDIALOG:  WMInitDialog( Msg );
  end;
end;

{ CCoolDialog }

constructor CCoolDialog.Create(uId: UINT; hParent: HWND; uStyle: UINT; dwValue: DWORD);
begin
  inherited Create(uId, hParent);
  m_uStyle := uStyle;
  m_bAlpha := Byte(dwValue);
  m_crTransparent := COLORREF(dwValue);
end;

procedure CCoolDialog.OnDestroy;
begin
  inherited OnDestroy;
end;

function CCoolDialog.OnInitDialog: BOOL;
begin
  {$IFDEF WIN2000}
  if ValidStyle then
  begin
    SetExLayeredWindow;
    case m_uStyle of
      LWA_ALPHA:
        SetLayeredWindowAttributes( hDlg, 0, m_bAlpha, m_uStyle );
      LWA_COLORKEY:
        SetLayeredWindowAttributes( hDlg, m_crTransparent, 0, m_uStyle );
    end;
  end;
  {$ENDIF}
  Result := inherited OnInitDialog;

end;
{$IFDEF WIN2000}
procedure CCoolDialog.SetAlpha(bValue: Byte);
begin
  if m_bAlpha <> bValue then
  begin
    m_uStyle := LWA_ALPHA;
    m_bAlpha := bValue;
    if IsWindow( hDlg ) then
      SetLayeredWindowAttributes( hDlg, 0, m_bAlpha, m_uStyle );
  end;
end;

procedure CCoolDialog.SetExLayeredWindow;
var
  dwStyle: DWORD;
begin
  dwStyle := GetWindowLong( hDlg, GWL_EXSTYLE );
  if dwStyle and WS_EX_LAYERED = 0 then
    SetWindowLong( hDlg, GWL_EXSTYLE, dwStyle or WS_EX_LAYERED );
end;

procedure CCoolDialog.SetTransparent(crValue: COLORREF);
begin
  if m_crTransparent <> crValue then
  begin
    m_uStyle := LWA_COLORKEY;
    m_crTransparent := crValue;
    if IsWindow( hDlg ) then
      SetLayeredWindowAttributes( hDlg, m_crTransparent, 0, m_uStyle );
  end;
end;

function CCoolDialog.ValidStyle: Bool;
begin
  Result := (m_uStyle = LWA_ALPHA) or (m_uStyle = LWA_COLORKEY);
end;
{$ENDIF}
procedure CCoolDialog.WMLButtonDown(var Msg: TMessage);
begin
  if Msg.WParam and MK_LBUTTON <> 0 then
  begin
    ReleaseCapture;
    SendMessage( hDlg, WM_SYSCOMMAND, SC_DRAGMOVE, 0 );
  end;
end;

procedure CCoolDialog.WndProc(var Msg: TMessage);
begin
  case Msg.Msg of
    WM_LBUTTONDOWN: WMLButtonDown( Msg );
  else
    inherited WndProc( Msg );
  end;
end;

{ CAboutDialog }

constructor CAboutDialog.Create(hWndMain: HWND; bAlpha: Byte);
begin
  {$IFDEF WIN2000}
  inherited Create( IDD_ABOUT_DLG, hWndMain, LWA_ALPHA, bAlpha );
  {$ELSE}
  inherited Create( IDD_ABOUT_DLG, hWndMain, 0, bAlpha );
  {$ENDIF}
end;

procedure CAboutDialog.OnDestroy;
begin
  DeleteObject( m_hFontProduct );
  DeleteObject( m_hbrBack );
  inherited OnDestroy;
end;

{ GetProductValue }
function GetProductValue(const strKey: string): string;
var
  Key: HKEY;
  dwSize: DWORD;
  strRootKey: string;
begin
  Result := '';
  strRootKey := 'SOFTWARE\Microsoft\Windows NT\CurrentVersion';
  if RegOpenKeyEx(HKEY_LOCAL_MACHINE, PChar(strRootKey), 0, KEY_READ, Key) = ERROR_SUCCESS then
  begin
    dwSize := 0;
    if RegQueryValueEx(Key, PChar(strKey), nil, nil, nil, @dwSize) = ERROR_SUCCESS then
    begin
      SetLength(Result, dwSize);
      if RegQueryValueEx(Key, PChar(strKey), nil, nil, PByte(Result), @dwSize) = ERROR_SUCCESS then
        if dwSize > 0 then SetLength( Result, dwSize-1 );
    end;
  end;
end;

function CAboutDialog.OnInitDialog: BOOL;
var
  ncm: NONCLIENTMETRICS;
  lf: LOGFONT;
  dc: HDC;
  osvi: OSVERSIONINFO;
  strId, strVer: string;
begin
  Result := inherited OnInitDialog;
  ncm.cbSize := SizeOf( ncm );
  SystemParametersInfo( SPI_GETNONCLIENTMETRICS, 0, @ncm, 0 );
  System.Move( ncm.lfMessageFont, lf, SizeOf( LOGFONT ) );
  lf.lfWeight := FW_BOLD;
  //lstrcpy( lf.lfFaceName, 'Times New Roman' );
  lstrcpy( lf.lfFaceName, 'Impact' );
  dc := GetDC( 0 );
  lf.lfHeight := 0 - GetDeviceCaps( dc, LOGPIXELSY ) * 24 div 72;
  m_hFontProduct := CreateFontIndirect( lf );
  ReleaseDC( 0, dc );
  m_hbrBack := CreateSolidBrush( GetSysColor( COLOR_INFOBK ) );
  SendDlgItemMessage( hDlg, 1000, WM_SETFONT, m_hFontProduct, 1 );

  FillChar( osvi, SizeOf(osvi), 0 );
  osvi.dwOSVersionInfoSize := SizeOf(osvi);
  if GetVersionEx( osvi ) then
  begin
    case osvi.dwPlatformId of
      VER_PLATFORM_WIN32_WINDOWS: strId := 'Windows 98';
      VER_PLATFORM_WIN32_NT: strId := 'Windows 2000';
    end;
    strVer := Format( '%s %d.%d (Build %d: %s)'#13#10#13#10 +
      'This product is licensed to:'#13#10'%s'#13#10'%s',
      [strId, osvi.dwMajorVersion,
      osvi.dwMinorVersion,
      osvi.dwBuildNumber and $FFFF,
      osvi.szCSDVersion,
      GetProductValue('RegisteredOwner'),
      GetProductValue('RegisteredOrganization')] );
    SetDlgItemText( hDlg, 1002, PChar(strVer) );
  end;

end;

procedure CAboutDialog.WMCtlColorDlg(var Msg: TMessage);
begin
  if HWND(Msg.lParam) = GetDlgItem( m_hDlg, 1000 ) then
    SetTextColor( HDC( Msg.wParam ), RGB(0, 0, 128) );
  SetBkMode( HDC( Msg.wParam ), TRANSPARENT );
  Msg.Result := m_hbrBack;
end;

procedure CAboutDialog.WndProc(var Msg: TMessage);
begin
  case Msg.Msg of
    WM_CTLCOLORSTATIC,
    WM_CTLCOLORDLG: WMCtlColorDlg( Msg );
  else
    inherited WndProc( Msg );
  end;
end;

end.
