unit Archives;

interface

uses Windows, Messages, Contnrs, UnRAR, ZLib, Cabinets;

type

  PRARHeaderDataEx = ^TRARHeaderDataEx;
  TRARHeaderDataEx = RARHeaderDataEx;
  
  { Forward declarations }
  TCustomArchiveFile = class;
  TArchiveFile = class;

  { TCustomArchiveFileItem }
  TCustomArchiveFileItem = class(TObject)
  private
    FArchiveFile: TArchiveFile;
    FFileName: string;
    FFileType: string;
    FDateTime: Cardinal;
    FSize: Cardinal;
    FRatio: Integer;
    FPackedSize: Cardinal;
    FCRC32: Cardinal;
    FAttribute: Cardinal;
    FPath: string;
    FImage: Integer;

    function GetRatio: Integer;
    function GetImage: Integer;
    function GetFileType: string;

    property Image: Integer read GetImage;
    property FileName: string read FFileName write FFileName;
    property FileType: string read GetFileType;
    property DateTime: Cardinal read FDateTime write FDateTime;
    property Size: Cardinal read FSize write FSize;
    property Ratio: Integer read GetRatio;
    property PackedSize: Cardinal read FPackedSize write FPackedSize;
    property CRC32: Cardinal read FCRC32 write FCRC32;
    property Attribute: Cardinal read FAttribute write FAttribute;
    property Path: string read FPath write FPath;
    function GetDateTimeAsString: string;
    function GetSizeAsString: string;
    function GetRatioAsString: string;
    function GetPackedSizeAsString: string;
    function GetCRC32AsString: string;
    function GetAttributeAsString: string;
    property DateTimeAsString: string read GetDateTimeAsString;
    property SizeAsString: string read GetSizeAsString;
    property RatioAsString: string read GetRatioAsString;
    property PackedSizeAsString: string read GetPackedSizeAsString;
    property CRC32AsString: string read GetCRC32AsString;
    property AttributeAsString: string read GetAttributeAsString;
    property ArchiveFile: TArchiveFile read FArchiveFile;
  protected

  public
    constructor Create(ArchiveFile: TArchiveFile);
    destructor Destroy; override;
    procedure Extract(const DestDir: string); virtual;
  end;

  { TArchiveFileItem }
  TArchiveFileItem = class(TCustomArchiveFileItem)
  private

  protected

  public
    property FileName;
    property FileType;
    property DateTime;
    property Size;
    property Ratio;
    property PackedSize;
    property CRC32;
    property Attribute;
    property Path;
    property Image;
    property DateTimeAsString;
    property SizeAsString;
    property RatioAsString;
    property PackedSizeAsString;
    property CRC32AsString;
    property AttributeAsString;
    property ArchiveFile;
  end;

  { TRarArchiveFileItem }
  TRarArchiveFileItem = class(TArchiveFileItem)
  private
    FHeader: TRARHeaderDataEx;
    procedure SetHeader(Header: TRARHeaderDataEx);
  protected

  public
    procedure Extract(const DestDir: string); override;
    property Header: TRARHeaderDataEx read FHeader write SetHeader;
  end;

  { TZipArchiveFileItem }
  TZipArchiveFileItem = class(TArchiveFileItem)
  private
    FHeader: TZipFileHeader;
    procedure SetHeader(Header: TZipFileHeader);
  protected

  public
    procedure Extract(const DestDir: string); override;
    property Header: TZipFileHeader read FHeader write SetHeader;
  end;

  { TCabArchiveFileItem }
  TCabArchiveFileItem = class(TArchiveFileItem)
  private
    //FHeader: TCabFileHeader;
    //procedure SetHeader(Header: TZipFileHeader);
  protected

  public
    procedure Extract(const DestDir: string); override;
    //property Header: TZipFileHeader read FHeader write SetHeader;
  end;

  TAddItemEvent = procedure (Sender: TCustomArchiveFile; Item: TArchiveFileItem) of object;
  TExtractItemEvent = procedure (Sender: TCustomArchiveFile; Item: TArchiveFileItem) of object;

  { TCustomArchiveFile }
  TCustomArchiveFile = class(TObject)
  private
    FActive: Boolean;
    FFileList: TObjectList;
    FArchiveName: string;
    FAddItemEvent: TAddItemEvent;
    FExtractItemEvent: TExtractItemEvent;
    FSize: Cardinal;
    FPackedSize: Cardinal;
    function GetRatio: Integer;
    function GetItem(Index: Integer): TArchiveFileItem;
    function GetCount: Integer;
    procedure SetArchiveName(Value: string);
    procedure SetActive(Value: Boolean);
    property Active: Boolean read FActive write SetActive;
    property Items[Index: Integer]: TArchiveFileItem read GetItem; default;
    property Count: Integer read GetCount;
    property ArchiveName: string read FArchiveName write SetArchiveName;
    property OnAddItem: TAddItemEvent read FAddItemEvent write FAddItemEvent;
    property OnExtractItem: TExtractItemEvent read FExtractItemEvent write FExtractItemEvent;
    property Size: Cardinal read FSize;
    property PackedSize: Cardinal read FPackedSize;
    property Ratio: Integer read GetRatio;
    property FileList: TObjectList read FFileList;
  protected

  public
    constructor Create;
    destructor Destroy; override;
    function Add(Item: TArchiveFileItem): Integer;
    function LoadList: Boolean; virtual;
    procedure ExtractAll(const DestDir: string); virtual;
    procedure Clear;
    procedure Close;
    procedure Open;
  end;

  { TArchiveFile }
  TArchiveFile = class(TCustomArchiveFile)
  private

  protected

  public
    property Active;
    property Items;
    property Count;
    property ArchiveName;
    property OnAddItem;
    property OnExtractItem;
    property Size;
    property PackedSize;
    property Ratio;
    property FileList;
  end;

  { TRarArchiveFile }
  TRarArchiveFile = class(TArchiveFile)
  private

  protected

  public
    function LoadList: Boolean; override;
    procedure ExtractAll(const DestDir: string); override;
  end;

  { TZipArchiveFile }
  TZipArchiveFile = class(TArchiveFile)
  private
    FHandle: THandle;
    function GetCentralDir(var EndOfCentralDir: TZipCentralDirectory): Boolean;
  protected

  public
    constructor Create;
    destructor Destroy; override;
    function LoadList: Boolean; override;
    procedure ExtractAll(const DestDir: string); override;
    property Handle: THandle read FHandle;
  end;

  { TCabArchiveFile }
  TCabArchiveFile = class(TArchiveFile)
  private

  protected

  public
    function LoadList: Boolean; override;
    procedure ExtractAll(const DestDir: string); override;
  end;

implementation

uses ShellApi, SysUtils;

function UnixPathToDosPath(const Path: string): string;
var
  I: Integer;
begin
  Result := Path;
  for I := 1 to Length(Result) do
    if Result[I] = '/' then Result[I] := '\';
end;

{ TCustomArchiveFileItem }

constructor TCustomArchiveFileItem.Create(ArchiveFile: TArchiveFile);
begin
  FArchiveFile := ArchiveFile;
  FImage := -1;
  FRatio := -1;
  FPackedSize := Cardinal(-1);
  FCRC32 := Cardinal(-1);
end;

destructor TCustomArchiveFileItem.Destroy;
begin
  inherited Destroy;
end;

procedure TCustomArchiveFileItem.Extract(const DestDir: string);
begin

end;

function TCustomArchiveFileItem.GetAttributeAsString: string;
begin
  Result := '';
  if FAttribute and FILE_ATTRIBUTE_READONLY <> 0 then
    Result := Result + 'R';
  if FAttribute and FILE_ATTRIBUTE_HIDDEN <> 0 then
    Result := Result + 'H';
  if FAttribute and FILE_ATTRIBUTE_SYSTEM <> 0 then
    Result := Result + 'S';
  if FAttribute and FILE_ATTRIBUTE_DIRECTORY <> 0 then
    Result := Result + 'D';
  if FAttribute and FILE_ATTRIBUTE_ARCHIVE <> 0 then
    Result := Result + 'A';
end;

function TCustomArchiveFileItem.GetCRC32AsString: string;
begin
  if FCRC32 <> Cardinal(-1) then
    Result := Format('%.8x', [FCRC32])
  else Result := '';
end;

function TCustomArchiveFileItem.GetDateTimeAsString: string;
begin
  Result := FormatDateTime('dd.mm.yyyy hh:mm',
    FileDateToDateTime(FDateTime));
end;

function TCustomArchiveFileItem.GetFileType: string;
var
  FileInfo: _SHFILEINFO;
begin
  if FImage = -1 then
  begin
    SHGetFileInfo(PChar(FFileName),
      0,
      FileInfo,
      SizeOf(FileInfo),
      SHGFI_SYSICONINDEX or SHGFI_LARGEICON or
      SHGFI_USEFILEATTRIBUTES or SHGFI_TYPENAME);
    FImage := FileInfo.iIcon;
    FFileType := FileInfo.szTypeName;
  end;
  Result := FFileType;
end;

function TCustomArchiveFileItem.GetImage: Integer;
var
  FileInfo: _SHFILEINFO;
begin
  if FImage = -1 then
  begin
    SHGetFileInfo(PChar(FFileName),
      0,
      FileInfo,
      SizeOf(FileInfo),
      SHGFI_SYSICONINDEX or SHGFI_LARGEICON or
      SHGFI_USEFILEATTRIBUTES or SHGFI_TYPENAME);
    FImage := FileInfo.iIcon;
    FFileType := FileInfo.szTypeName;
  end;
  Result := FImage;
end;

function TCustomArchiveFileItem.GetPackedSizeAsString: string;
begin
  if FPackedSize <> Cardinal(-1) then
    Result := Format('%.0n', [FPackedSize * 1.0])
  else
    Result := '?';
end;

function TCustomArchiveFileItem.GetRatio: Integer;
begin
  if FRatio = -1 then
  begin
    if FSize <> 0 then FRatio := 100 - Round(FPackedSize / FSize * 100)
    else FRatio := 0;
  end;
  if (FRatio < 0) or (FRatio > 100) then FRatio := 0;
  Result := FRatio;
end;

function TCustomArchiveFileItem.GetRatioAsString: string;
begin
  Result := Format('%u%%', [Ratio]);
end;

function TCustomArchiveFileItem.GetSizeAsString: string;
begin
  Result := Format('%.0n', [FSize * 1.0]);
end;

{ TRarArchiveFileItem }

procedure TRarArchiveFileItem.Extract(const DestDir: string);
begin
  
end;

procedure TRarArchiveFileItem.SetHeader(Header: TRARHeaderDataEx);
begin
  System.Move(Header, FHeader, SizeOf(TRARHeaderDataEx));
end;

{ TZipArchiveFileItem }

function DirectoryExists(const Name: string): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(Name));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;

procedure ForceDirectories(Dir: string);
begin
  if (AnsiLastChar(Dir) <> nil) and (AnsiLastChar(Dir)^ = '\') then
    Delete(Dir, Length(Dir), 1);
  if (Length(Dir) < 3) or DirectoryExists(Dir)
    or (ExtractFilePath(Dir) = Dir) then Exit; // avoid 'xyz:\' problem.
  ForceDirectories(ExtractFilePath(Dir));
  CreateDir(Dir);
end;

function Min(A, B: Integer): Integer;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;

const
  BUFFER_IN_SIZE  = (64*1024);
  BUFFER_OUT_SIZE = (128*1024);

type
  PZStreamRec = ^TZStreamRec;
(*
function ExtractZipFile(Archive: TArchiveFile; const DestDir: string;
  ArchiveFile: TZipArchiveFile): Boolean;
const
  MAX_WBITS = 15;
var
  Stream: TZStreamRec;
  DestFile: THANDLE;
  Local: TZipLocalFileHeader;
  nRead, nWrite: DWORD;
  nCompressed, nUncompressed: DWORD;
  BufferIn, BufferOut: PChar;
  nInflate: Integer;
  FullPath: string;
begin

  Result := False;
  SetFilePointer(Archive.Handle, ArchiveFile.Header.RelativeOffset, nil, FILE_BEGIN);
  ReadFile( Archive.Handle, Local, SizeOf(TZipLocalFileHeader), nRead, nil );
  if (Local.Method <> Z_DEFLATED) and (Local.Method <> 0) then Exit;

  FullPath := ArchiveFile.Path;
  if DestDir[Length(DestDir)] = '\' then
    FullPath := DestDir + FullPath
  else
    FullPath := DestDir + '\' + FullPath;
  ForceDirectories(FullPath);
  FullPath := FullPath + ArchiveFile.FileName;

  DestFile := CreateFile( PChar(FullPath), GENERIC_WRITE, 0, nil, CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL, 0 );

  if DestFile = INVALID_HANDLE_VALUE then Exit;

  SetFilePointer( Archive.Handle, Local.NameLength + Local.ExtraLength,
    nil, FILE_CURRENT );

  FillChar( Stream, sizeof(TZStreamRec), 0 );

  BufferIn := AllocMem(BUFFER_IN_SIZE);
  BufferOut := AllocMem(BUFFER_OUT_SIZE);
  nCompressed := 0;
  nUncompressed := 0;
  Stream.zalloc := zlibAllocMem;
  Stream.zfree := zlibFreeMem;
  Stream.next_in := BufferIn;
  Stream.avail_in := 0;

  try
    DCheck(InflateInit2(Stream, -MAX_WBITS));
    try
      while (nCompressed < Local.CompSize) or (nUncompressed < Local.UnCompSize) do
      begin

        if Stream.avail_in = 0 then
        begin
          Stream.avail_in := Min(Local.CompSize - nCompressed, BUFFER_IN_SIZE);
          Stream.next_in := BufferIn;
          ReadFile(Archive.Handle, BufferIn^, Stream.avail_in, nRead, nil);
          if Integer(nRead) <> Stream.avail_in then Break;
          Inc(nCompressed, nRead);
        end;
        Stream.avail_out := BUFFER_OUT_SIZE;
        Stream.next_out	:= BufferOut;
        nInflate := Inflate(Stream, Z_SYNC_FLUSH);
        if (nInflate = Z_OK) or (nInflate = Z_STREAM_END) then
        begin
          if Stream.avail_out < BUFFER_OUT_SIZE then
          begin
            nWrite := BUFFER_OUT_SIZE - Stream.avail_out;
            WriteFile(DestFile, BufferOut^, nWrite, nWrite, nil);
            if (Integer(nWrite) <> BUFFER_OUT_SIZE - Stream.avail_out) then Break;
            Inc(nUncompressed, nWrite);
          end;
        end
        else Break;
      end;
    finally
      InflateEnd(Stream);
    end;
  except
    FreeMem(BufferIn);
    FreeMem(BufferOut);
    raise;
  end;

  CloseHandle(DestFile);

end;
*)

procedure TZipArchiveFileItem.Extract(const DestDir: string);
const
  MAX_WBITS = 15;
var
  Stream: TZStreamRec;
  DestFile: THANDLE;
  Local: TZipLocalFileHeader;
  nRead, nWrite: DWORD;
  nCompressed, nUncompressed: DWORD;
  BufferIn, BufferOut: PChar;
  nInflate: Integer;
  FullPath: string;
  DateTime, lf: FILETIME;
  hFile: THandle;
begin

  hFile := TZipArchiveFile(ArchiveFile).Handle;
  SetFilePointer( hFile, Header.RelativeOffset, nil, FILE_BEGIN);
  ReadFile( hFile, Local, SizeOf(TZipLocalFileHeader), nRead, nil );
  if (Local.Method <> Z_DEFLATED) and (Local.Method <> 0) then Exit;

  FullPath := Path;
  if DestDir[Length(DestDir)] = '\' then
    FullPath := DestDir + FullPath
  else
    FullPath := DestDir + '\' + FullPath;
  ForceDirectories(FullPath);
  FullPath := FullPath + FileName;

  DestFile := CreateFile( PChar(FullPath), GENERIC_WRITE, 0, nil, CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL, 0 );

  if DestFile = INVALID_HANDLE_VALUE then Exit;

  SetFilePointer( hFile, Local.NameLength + Local.ExtraLength,
    nil, FILE_CURRENT );

  FillChar( Stream, sizeof(TZStreamRec), 0 );

  BufferIn := AllocMem(BUFFER_IN_SIZE);
  BufferOut := AllocMem(BUFFER_OUT_SIZE);
  nCompressed := 0;
  nUncompressed := 0;
  Stream.zalloc := zlibAllocMem;
  Stream.zfree := zlibFreeMem;
  Stream.next_in := BufferIn;
  Stream.avail_in := 0;

  try
    DCheck(InflateInit2(Stream, -MAX_WBITS));
    try
      while (nCompressed < Local.CompSize) or (nUncompressed < Local.UnCompSize) do
      begin

        if Stream.avail_in = 0 then
        begin
          Stream.avail_in := Min(Local.CompSize - nCompressed, BUFFER_IN_SIZE);
          Stream.next_in := BufferIn;
          ReadFile(TZipArchiveFile(ArchiveFile).Handle,
            BufferIn^, Stream.avail_in, nRead, nil);
          if Integer(nRead) <> Stream.avail_in then Break;
          Inc(nCompressed, nRead);
        end;
        Stream.avail_out := BUFFER_OUT_SIZE;
        Stream.next_out	:= BufferOut;
        nInflate := Inflate(Stream, Z_SYNC_FLUSH);
        if (nInflate = Z_OK) or (nInflate = Z_STREAM_END) then
        begin
          if Stream.avail_out < BUFFER_OUT_SIZE then
          begin
            nWrite := BUFFER_OUT_SIZE - Stream.avail_out;
            WriteFile(DestFile, BufferOut^, nWrite, nWrite, nil);
            if (Integer(nWrite) <> BUFFER_OUT_SIZE - Stream.avail_out) then Break;
            Inc(nUncompressed, nWrite);
          end;
        end
        else Break;
      end;
    finally
      InflateEnd(Stream);
    end;
  except
    FreeMem(BufferIn);
    FreeMem(BufferOut);
    raise;
  end;

  if DosDateTimeToFileTime( FHeader.Date, FHeader.Time, DateTime ) then
  begin
    LocalFileTimeToFileTime( DateTime, lf );
    SetFileTime( DestFile, @lf, nil, @lf );
  end;

  CloseHandle(DestFile);

end;

procedure TZipArchiveFileItem.SetHeader(Header: TZipFileHeader);
begin
  System.Move(Header, FHeader, SizeOf(TZipFileHeader));
end;

{ TCustomArchiveFile }

function TCustomArchiveFile.Add(Item: TArchiveFileItem): Integer;
begin
  Result := FFileList.Add(Item);
  Inc(FSize, Item.Size);
  Inc(FPackedSize, Item.PackedSize);
  if Assigned(FAddItemEvent) then
    FAddItemEvent(Self, Item);
end;

procedure TCustomArchiveFile.Clear;
begin
  FFileList.Clear;
end;

procedure TCustomArchiveFile.Close;
begin
  Clear;
  FActive := False;
end;

constructor TCustomArchiveFile.Create;
begin
  FFileList := TObjectList.Create;
  FArchiveName := '';
end;

destructor TCustomArchiveFile.Destroy;
begin
  Close;
  FFileList.Free;
  inherited Destroy;
end;

procedure TCustomArchiveFile.ExtractAll(const DestDir: string);
begin
  // Vorsicht! Abstract method
end;

function TCustomArchiveFile.GetCount: Integer;
begin
  Result := FFileList.Count;
end;

function TCustomArchiveFile.GetItem(Index: Integer): TArchiveFileItem;
begin
  Result := TArchiveFileItem(FFileList[Index]);
end;

function TCustomArchiveFile.GetRatio: Integer;
begin
  if FSize <> 0 then Result := Trunc(FPackedSize / FSize * 100)
  else Result := 0;
end;

function TCustomArchiveFile.LoadList: Boolean;
begin
  Result := False; // Vorsicht! Abstract method
end;

procedure TCustomArchiveFile.Open;
begin
  if not FActive then begin
    FActive := LoadList;
  end;
end;

procedure TCustomArchiveFile.SetActive(Value: Boolean);
begin
  if FActive <> Value then
  begin
    if Value then Open
    else Close;
  end;
end;

procedure TCustomArchiveFile.SetArchiveName(Value: string);
begin
  if not FActive then begin
    FArchiveName := Value;
  end;
end;

{ TRarArchiveFile }

const
  _EXTRACT = 0;
  _TEST    = 1;
  _PRINT   = 2;

function CallbackProc(msg: UINT; UserData, P1, P2: integer) :integer; stdcall;
begin
  Result := 0;
  case msg of
    UCM_CHANGEVOLUME:
      if (P2 = RAR_VOL_ASK) then begin
      end;
    UCM_NEEDPASSWORD:
      begin
      end;
    UCM_PROCESSDATA:
      begin
      end;
  end;
end;

function ExtractRarArchive(const ArcName: string; Mode: Integer; const DestDir: string): Integer;
var
  hArcData: THandle;
  RHCode, PFCode: Integer;
  CmtBuf: array[0..Pred(16384)] of Char;
  HeaderData: RARHeaderData;
  OpenArchiveData: RAROpenArchiveDataEx;
  Operation: Integer;
  SaveDir: string;
begin
  FillChar(OpenArchiveData, SizeOf(OpenArchiveData), 0);
  OpenArchiveData.ArcName := PChar(ArcName);
  OpenArchiveData.CmtBuf := @CmtBuf;
  OpenArchiveData.CmtBufSize := SizeOf(CmtBuf);
  OpenArchiveData.OpenMode := RAR_OM_EXTRACT;
  hArcData := RAROpenArchiveEx(OpenArchiveData);

  if (OpenArchiveData.OpenResult <> 0) then
  begin
    Result := OpenArchiveData.OpenResult;
    Exit;
  end;

  SaveDir := GetCurrentDir;

  SetCurrentDir( DestDir );

  if (OpenArchiveData.CmtState = 1) then;

  RARSetCallback(hArcData, CallbackProc, Integer(@Mode));

  HeaderData.CmtBuf := nil;

  repeat
    RHCode := RARReadHeader(hArcData, HeaderData);
    if RHCode <> 0 then
      Break;

    case Mode of
      _EXTRACT:;
      _TEST:;
      _PRINT:;
    end;

    if Mode = _EXTRACT then
      Operation := RAR_EXTRACT
    else
      Operation := RAR_TEST;
    PFCode := RARProcessFile(hArcData, Operation, nil, nil);

    if (PFCode = 0) then
    else begin
      Break;
    end;
  until False;

  if (RHCode = ERAR_BAD_DATA) then;

  RARCloseArchive(hArcData);

  Result := RHCode;

  SetCurrentDir( SaveDir );

end;

procedure TRarArchiveFile.ExtractAll(const DestDir: string);
var
  hArcData: THandle;
  RHCode, PFCode: Integer;
  CmtBuf: array[0..Pred(16384)] of Char;
  HeaderData: RARHeaderData;
  OpenArchiveData: RAROpenArchiveDataEx;
  Operation: Integer;
  SaveDir: string;
  Mode: Integer;
  Index: Integer;
begin
  //ExtractRarArchive(ArchiveName, _EXTRACT, DestDir);
  if not Active then Exit;
  FillChar(OpenArchiveData, SizeOf(OpenArchiveData), 0);
  OpenArchiveData.ArcName := PChar(ArchiveName);
  OpenArchiveData.CmtBuf := @CmtBuf;
  OpenArchiveData.CmtBufSize := SizeOf(CmtBuf);
  OpenArchiveData.OpenMode := RAR_OM_EXTRACT;
  hArcData := RAROpenArchiveEx(OpenArchiveData);

  if (OpenArchiveData.OpenResult <> 0) then
  begin
    Exit;
  end;

  SaveDir := GetCurrentDir;
  SetCurrentDir( DestDir );

  if (OpenArchiveData.CmtState = 1) then;

  Mode := _EXTRACT;

  RARSetCallback(hArcData, CallbackProc, Integer(@Mode));

  HeaderData.CmtBuf := nil;

  Index := 0;

  repeat
    RHCode := RARReadHeader(hArcData, HeaderData);
    if RHCode <> 0 then
      Break;

    case Mode of
      _EXTRACT:;
      _TEST:;
      _PRINT:;
    end;

    if Mode = _EXTRACT then
      Operation := RAR_EXTRACT
    else
      Operation := RAR_TEST;
    PFCode := RARProcessFile(hArcData, Operation, nil, nil);

    if (PFCode = 0) then
    else begin
      Break;
    end;
    
    if HeaderData.FileAttr and FILE_ATTRIBUTE_DIRECTORY = 0 then
    begin
      if Assigned(OnExtractItem) then
        OnExtractItem( Self, Items[Index] );
      Inc(Index);
    end;

  until False;

  if (RHCode = ERAR_BAD_DATA) then;

  RARCloseArchive(hArcData);

  SetCurrentDir( SaveDir );

end;

function TRarArchiveFile.LoadList: Boolean;
var
  hArcData: THandle;
  RHCode, PFCode: Integer;
  CmtBuf: array[0..Pred(16384)] of Char;
  HeaderData: RARHeaderDataEx;
  OpenArchiveData: RAROpenArchiveDataEx;
  ArchiveFileItem: TRarArchiveFileItem;
begin
  Result := False;
  FillChar(OpenArchiveData, SizeOf(OpenArchiveData), 0);
  OpenArchiveData.ArcName := PChar(ArchiveName);
  OpenArchiveData.CmtBuf := @CmtBuf;
  OpenArchiveData.CmtBufSize := SizeOf(CmtBuf);
  OpenArchiveData.OpenMode := RAR_OM_LIST;
  hArcData := RAROpenArchiveEx(OpenArchiveData);
  if (OpenArchiveData.OpenResult <> 0) then
  begin
    Exit;
  end;
  PFCode := -1;
  RARSetCallback (hArcData, CallbackProc, 0);
  HeaderData.CmtBuf := @CmtBuf;
  HeaderData.CmtBufSize := SizeOf(CmtBuf);
  repeat
    RHCode := RARReadHeaderEx(hArcData, HeaderData);
    if RHCode <> 0 then
      Break;
    if HeaderData.FileAttr and FILE_ATTRIBUTE_DIRECTORY = 0 then
    begin
      ArchiveFileItem := TRarArchiveFileItem.Create(Self);
      with ArchiveFileItem do
      begin
        Header := HeaderData;
        FileName := ExtractFileName(HeaderData.FileName);
        Size := HeaderData.UnpSize;
        PackedSize := HeaderData.PackSize;
        DateTime := HeaderData.FileTime;
        CRC32 := HeaderData.FileCRC;
        Attribute := HeaderData.FileAttr;
        Path := ExtractFilePath(HeaderData.FileName);
       end;
      Add(ArchiveFileItem);
    end;
    PFCode := RARProcessFile(hArcData, RAR_SKIP, nil, nil);
    if (PFCode <> 0) then
    begin
      Break;
    end;
  until False;
  RARCloseArchive(hArcData);
  Result := ( (RHCode = 0) or (RHCode = ERAR_END_ARCHIVE) ) and
    (OpenArchiveData.OpenResult = 0) and (PFCode = 0);
end;

{ TZipArchiveFile }

constructor TZipArchiveFile.Create;
begin
  inherited Create;
  FHandle := INVALID_HANDLE_VALUE;
end;

destructor TZipArchiveFile.Destroy;
begin
  if FHandle <> INVALID_HANDLE_VALUE then
    CloseHandle(FHandle);
  inherited Destroy;
end;

procedure TZipArchiveFile.ExtractAll(const DestDir: string);
var
  Index: Integer;
begin
  for Index := 0 to Count-1 do
  begin
    Items[Index].Extract(DestDir);
    if Assigned(OnExtractItem) then
      OnExtractItem( Self, Items[Index] );
  end;
end;

function TZipArchiveFile.GetCentralDir(
  var EndOfCentralDir: TZipCentralDirectory): Boolean;
var
  CentralDir: PZipCentralDirectory;
  Buffer: array[0..4095] of Char;
  NumRead, Scan: DWORD;
  Signature: PDWORD;
begin
  Result := False;
  if FHandle <> INVALID_HANDLE_VALUE then
  begin
    CentralDir := nil;
    SetFilePointer(FHandle, -4096, nil, FILE_END);
    if not ReadFile(FHandle, Buffer, 4096, NumRead, nil) then Exit;
    if NumRead < SizeOf(TZipCentralDirectory) then Exit;
    for Scan := 4 to NumRead do
    begin
      Signature := PDWORD(Buffer + NumRead - Scan);
      if Signature^ = $06054b50 then
      begin
        CentralDir := PZipCentralDirectory(Signature);
        Result := True;
        Break;
      end;
    end;
    if CentralDir <> nil then
      System.Move(CentralDir^, EndOfCentralDir, SizeOf(TZipCentralDirectory));
  end;
end;

function TZipArchiveFile.LoadList: Boolean;
var
  FileHeader: TZipFileHeader;
  NumRead: Cardinal;
  Name, Extra, Comment: array[0..MAX_PATH-1] of Char;
  Item: TZipArchiveFileItem;
  StrDosName: string;
  EndCentDir: TZipCentralDirectory;
  CentralDir, Save: PChar;
  TotalFile: LongWord;
begin
  Result := False;
  if FHandle <> INVALID_HANDLE_VALUE then Exit;

  FHandle := CreateFile(PChar(ArchiveName), GENERIC_READ,
    FILE_SHARE_READ, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

  if FHandle = INVALID_HANDLE_VALUE then Exit;
  if not GetCentralDir(EndCentDir) then Exit;
  SetFilePointer(FHandle, EndCentDir.OffsetCentralDir, nil, FILE_BEGIN);
  CentralDir := AllocMem(EndCentDir.SizeOfCentralDir);
  if not ReadFile(FHandle, CentralDir^, EndCentDir.SizeOfCentralDir, NumRead, nil) then
  begin
    FreeMem(CentralDir);
    Exit;
  end;
  SetFilePointer(FHandle, EndCentDir.OffsetCentralDir, nil, FILE_BEGIN);
  Save := CentralDir;
  TotalFile := 0;
  while TotalFile < EndCentDir.TotalFileOnDir do
  begin
    if Save = nil then Break;
    Move(Save^, FileHeader, SizeOf(TZipFileHeader));

    if FileHeader.Signature <> $02014b50 then
    begin
      Break;
    end;
    Inc(Save, SizeOf(TZipFileHeader));

    FillChar(Name, SizeOf(Name), 0);
    Move(Save^, Name, FileHeader.NameLength);
    Inc(Save, FileHeader.NameLength);

    if FileHeader.ExtraLength > 0 then
    begin
      FillChar(Extra, SizeOf(Extra), 0);
      Move(Save^, Extra, FileHeader.ExtraLength);
      Inc(Save, FileHeader.NameLength);
    end;

    if FileHeader.CommentLength > 0 then
    begin
      FillChar(Comment, SizeOf(Comment), 0);
      Move(Save^, Comment, FileHeader.CommentLength);
      Inc(Save, FileHeader.CommentLength);
    end;

    if FileHeader.ExtAttribute and FILE_ATTRIBUTE_DIRECTORY = 0 then
    begin
      StrDosName := UnixPathToDosPath(Name);
      Item := TZipArchiveFileItem.Create(Self);
      with Item do
      begin
        Header := FileHeader;
        FileName := ExtractFileName(StrDosName);
        Size := FileHeader.UnCompSize;
        PackedSize := FileHeader.CompSize;
        DateTime := MAKELONG(FileHeader.Time, FileHeader.Date);
        CRC32 := FileHeader.CRC32;
        Attribute := FileHeader.IntAttribute;
        Path := ExtractFilePath(StrDosName);
      end;
      Add(Item);
    end;

    Inc(TotalFile);
    
  end;
  FreeMem(CentralDir);
  Result := True;
end;

{ TCabArchiveFile }

function FNAlloc(cb: Cardinal): Pointer; cdecl;
begin
  Result := AllocMem(cb);
end;

procedure FNFree(pv: Pointer); cdecl;
begin
  FreeMem(pv);
end;

function FNOpen(pszFile: LPSTR; oflag: Integer; pmode: Integer): Integer; cdecl;
begin
  {
  Result := CreateFile( pszFile, oflag, 0, nil, OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL, 0 );
  }
  Result := CreateFile( pszFile, GENERIC_READ,
    FILE_SHARE_READ, nil, OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL, 0 );
end;

function FNRead(hf: Integer; pv: Pointer; cb: UINT): UINT; cdecl;
begin
  ReadFile( hf, pv^, cb, Result, nil );
end;

function FNWrite(hf: Integer; pv: Pointer; cb: UINT): UINT; cdecl;
begin
  WriteFile( hf, pv, cb, Result, nil );
end;

function FNClose(hf: Integer): Integer; cdecl;
begin
  CloseHandle( hf );
  Result := FDIERROR_NONE;
end;

function FNSeek(hf: Integer; dist: LongInt; seektype: Integer): LongInt; cdecl;
begin
  Result := SetFilePointer( hf, dist, nil, seektype );
end;

function FNNotify(fdint: FDINOTIFICATIONTYPE; pfdin: PFDINOTIFICATION): Integer; cdecl;
var
  Item: TCabArchiveFileItem;
  Archive: TCabArchiveFile;
begin
  Result := 0;
  case fdint of
    fdintCABINET_INFO:
      begin
        Result := 0;
      end;
    fdintPARTIAL_FILE:
      begin
        Result := 0;
      end;
    fdintENUMERATE:
      begin
        Result := 0;
      end;
    fdintCOPY_FILE:
      begin
        if Assigned( pfdin^.pv ) then
        begin
          Archive := TCabArchiveFile(pfdin^.pv);
          Item := TCabArchiveFileItem.Create(Archive);
          with Item do
          begin
            FileName := ExtractFileName(pfdin^.psz1);
            Size := pfdin^.cb;
            DateTime := MAKELONG(pfdin^.time, pfdin^.date);
            Attribute := pfdin^.attribs;
            Path := ExtractFilePath(pfdin^.psz1);
          end;
          Archive.Add(Item);
        end;
        Result := 0;
      end;
    fdintCLOSE_FILE_INFO:
      begin
        Result := 0;
      end;
  end;
end;


procedure TCabArchiveFile.ExtractAll(const DestDir: string);
begin

end;

function TCabArchiveFile.LoadList: Boolean;
var
  perf: ERF;
  fdi: HFDI;
  Handle: THandle;
  fdici: FDICABINETINFO;
begin
  Result := False;
  fdi := FDICreate( FNAlloc, FNFree, FNOpen, FNRead, FNWrite, FNClose,
    FNSeek, cpu80386, @perf );
  Handle := CreateFile( PChar(ArchiveName), GENERIC_READ, 0, nil, OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL, 0 );
  if Handle <> INVALID_HANDLE_VALUE then
  begin
    if FDIIsCabinet( fdi, Handle, @fdici ) then
    begin
      CloseHandle( Handle );
      if FDICopy( fdi, PChar(ExtractFileName(ArchiveName)),
        PChar(ExtractFilePath(ArchiveName)), 0, FNnotify, nil, Self ) then
        Result := True;
    end;
  end;
  if Assigned(fdi) then
    FDIDestroy( fdi );
end;

{ TCabArchiveFileItem }

procedure TCabArchiveFileItem.Extract(const DestDir: string);
begin
  
end;

end.
