unit CDialogs;

interface

uses Windows, Messages, CommCtrl;//, SysUtils;//, Controls;

type
  TDialog = class(TObject)
  private
    FParent: HWND;
    FId: UINT;
    FHandle: HWND;
    FModal: Boolean;
  protected
    procedure OnCancel; virtual;
    procedure OnDestroy; virtual;
    function OnInitDialog: BOOL; virtual;
    procedure OnOK; virtual;
    procedure WMCommand(var Msg: TMessage); virtual;
    procedure WMInitDialog(var Msg: TMessage); virtual;
    procedure WMCtlColorBtn(var Msg: TMessage); virtual;
    procedure WMCtlColorDlg(var Msg: TMessage); virtual;
    procedure WMCtlColorEdit(var Msg: TMessage); virtual;
    procedure WMCtlColorListBox(var Msg: TMessage); virtual;
    procedure WMCtlColorStatic(var Msg: TMessage); virtual;
    procedure WndProc(var Msg: TMessage); virtual;
  public
    constructor Create(Parent: HWND; Id: UINT);
    destructor Destroy; override;
    procedure Show;
    function ShowModal: Integer;
    procedure ShowWindow(nShow: Integer);
    property Handle: HWND read FHandle write FHandle;
  end;

  TPropertiesDialog = class(TDialog)
  protected
    function OnInitDialog: BOOL; override;
    procedure WMInitDialog(var Msg: TMessage); override;
    procedure WndProc(var Msg: TMessage); override;
  public
    constructor Create(Parent: HWND);
    destructor Destroy; override;
  end;

  TControl = class;

  TControlClass = class of TControl;

  TMouseEvent = procedure (Sender: TControl) of object;

  TControl = class(TObject)
  private
    FHandle: HWND;
    FParent: HWND;
    FControlProc: Pointer;
    FControlId: Integer;
    FPopupMenu: HMENU;
    FFont: HFONT;
    FLeft: Integer;
    FTop: Integer;
    FBrush: HBRUSH;
    FColor: DWORD;
    FTextColor: DWORD;
    FCursor: HCURSOR;
    FExStyle: DWORD;
    FWndClass: WNDCLASS;
    FMouseEnter: TMouseEvent;
    FMouseLeave: TMouseEvent;
    procedure SetHandle(const Value: HWND);
    procedure SetColor(const Value: DWORD);
    procedure SetFont(const Value: HFONT);
    procedure SetTextColor(const Value: DWORD);
    procedure SetCursor(const Value: HCURSOR);
    function GetExStyle: DWORD;
    procedure SetExStyle(const Value: DWORD);
  protected
    procedure CMMouseEnter(var Msg: TMessage); virtual;
    procedure CMMouseLeave(var Msg: TMessage); virtual;
    procedure WMContextMenu(var Msg: TMessage); virtual;
    procedure WMMouseMove(var Msg: TMessage); virtual;
    procedure WMSetCursor(var Msg: TMessage); virtual;
    procedure WMMove(var Msg: TMessage); virtual;
    procedure WMSetFont(var Msg: TMessage); virtual;
    procedure WMWindowPosChanged(var Msg: TMessage); virtual;
    procedure WndProc(var Msg: TMessage); virtual;
  public
    constructor Create(Parent: HWND); virtual;
    destructor Destroy; override;
    procedure DefaultProc(var Msg: TMessage);
    function Perform(Msg: UINT; WParam, LParam: LongInt): LongInt;
    property Handle: HWND read FHandle write SetHandle;
    property Parent: HWND read FParent write FParent;
    property ControlProc: Pointer read FControlProc write FControlProc;
    property PopupMenu: HMENU read FPopupMenu write FPopupMenu;
    property ControlId: Integer read FControlId;
    property Brush: HBRUSH read FBrush;
    property Color: DWORD read FColor write SetColor;
    property TextColor: DWORD read FTextColor write SetTextColor;
    property Font: HFONT read FFont write SetFont;
    property Cursor: HCURSOR read FCursor write SetCursor;
    property ExStyle: DWORD read GetExStyle write SetExStyle;
    property MouseEnter: TMouseEvent read FMouseEnter write FMouseEnter;
    property MouseLeave: TMouseEvent read FMouseLeave write FMouseLeave;
  end;

  TButton = class(TControl)
  public
    constructor Create(Parent: HWND); override;
    destructor Destroy; override;
  end;

  TEdit = class(TControl)
  public
    constructor Create(Parent: HWND); override;
    destructor Destroy; override;
  end;

  TStaticText = class(TControl)
  public
    constructor Create(Parent: HWND); override;
    destructor Destroy; override;
  end;

  TListBox = class(TControl)
  public
    constructor Create(Parent: HWND); override;
    destructor Destroy; override;
    function Add(const S: string): LongInt;
    function GetDir(const FileSpec: string; Attrs: UINT): LongInt;
    function Clear: LongInt;
  end;

  TComboBox = class(TControl)
  end;

  TCheckBox = class(TControl)
  end;

  TRadioButton = class(TControl)
  end;

  TURLText = class(TStaticText)
  protected
    procedure CMMouseEnter(var Msg: TMessage); override;
    procedure CMMouseLeave(var Msg: TMessage); override;
  public
    constructor Create(Parent: HWND); override;
    destructor Destroy; override;
  end;

  TAppDialog = class(TDialog)
  private
    FFont: HFONT;
    FControlCount: Integer;
    //Static: TStaticText;
    function GetControl(Item: Integer): TControl;
    procedure SetProcControls;
  protected
    function OnInitDialog: BOOL; override;
    procedure OnFont(var Msg: TMessage);
    procedure OnAdd(var Msg: TMessage);
    procedure OnDir(var Msg: TMessage);
    procedure OnClear(var Msg: TMessage);
    procedure CMMouseEnter(var Msg: TMessage); virtual;
    procedure CMMouseLeave(var Msg: TMessage); virtual;
    procedure WMMouseMove(var Msg: TMessage); virtual;
    procedure WMCommand(var Msg: TMessage); override;
    procedure WMCtlColorBtn(var Msg: TMessage); override;
    procedure WMCtlColorDlg(var Msg: TMessage); override;
    procedure WMCtlColorEdit(var Msg: TMessage); override;
    procedure WMCtlColorListBox(var Msg: TMessage); override;
    procedure WMCtlColorStatic(var Msg: TMessage); override;
    procedure WndProc(var Msg: TMessage); override;
  public
    constructor Create(Id: UINT);
    destructor Destroy; override;
    function GetDlgItem(Id: Integer): TControl;
    function IncrementCount: Integer;
    procedure DoMouseEnter(Sender: TControl);
    procedure DoMouseLeave(Sender: TControl);
    property ControlCount: Integer read FControlCount;
    property Controls[Item: Integer]: TControl read GetControl;
  end;

  TApplication = class(TObject)
  private
    FMainWnd: HWND;
  protected

  public
    constructor Create;
    destructor Destroy; override;
    function InitInstance: Boolean; virtual;
    procedure Run; virtual;
  end;



implementation

function DialogProc( hDlg: HWND; uMsg: UINT; wParam, lParam: LongInt ): LRESULT; stdcall;
var
  Dialog: TDialog;
  Msg: TMessage;
begin

  Result := 0;

  Msg.Msg := uMsg;
  Msg.WParam := wParam;
  Msg.LParam := lParam;
  Msg.Result := Integer( Result );

  //Dialog := TDialog( GetWindowLong( hDlg, GWL_USERDATA ) );
  Dialog := TDialog( GetProp( hDlg, 'Dialog_Object' ) );

  if not Assigned( Dialog ) then
  begin

    case uMsg of
      WM_INITDIALOG:
        begin
          Dialog := TDialog( lParam );
          Dialog.Handle := hDlg;
          SetProp( hDlg, 'Dialog_Object', THANDLE( Dialog ) );
          Dialog.WndProc( Msg );
        end;
      WM_DESTROY:
        begin
          RemoveProp( hDlg, 'Dialog_Object' );
        end;
      WM_NCDESTROY:
        begin
          RemoveProp( hDlg, 'Dialog_Object' );
        end;
    end;
    {
    if uMsg = WM_INITDIALOG then
    begin
      Dialog := TDialog( lParam );
      Dialog.Handle := hDlg;
      SetWindowLong( hDlg, GWL_USERDATA, LongInt( Dialog ) );
      Dialog.WndProc( Msg );
    end;
    }
  end
  else Dialog.WndProc( Msg );

  Result := Msg.Result;

end;

{ TDialog }

constructor TDialog.Create(Parent: HWND; Id: UINT);
begin
  FParent := Parent;
  FId := Id;
  FHandle := 0;
  FModal := False;
end;

destructor TDialog.Destroy;
begin
  OnDestroy;
  if IsWindow( FHandle ) then
    Windows.DestroyWindow( FHandle );
  inherited Destroy;
end;

procedure TDialog.OnCancel;
begin
  EndDialog( FHandle, IDCANCEL );
end;

procedure TDialog.OnDestroy;
begin
  RemoveProp( FHandle, 'Dialog_Object' );
end;

function TDialog.OnInitDialog: BOOL;
begin
  Result := False;
end;

procedure TDialog.OnOK;
begin
  EndDialog( FHandle, IDOK );
end;

procedure TDialog.Show;
begin
  if not IsWindow( FHandle ) then
  begin
    CreateDialogParam( SysInit.HInstance, MakeIntResource( FId ),
      FParent, @DialogProc, Integer( Self ) );
  end;
  ShowWindow( SW_SHOW );
end;

function TDialog.ShowModal: Integer;
var
  hWndFocus: HWND;
begin
  hWndFocus := GetFocus;
  Result := DialogBoxParam( SysInit.HInstance, MakeIntResource( FId ),
    FParent, @DialogProc, Integer( Self ) );
  SetFocus( hWndFocus );
end;

procedure TDialog.ShowWindow(nShow: Integer);
begin
  Windows.ShowWindow( FHandle, nShow );
end;

procedure TDialog.WMCommand(var Msg: TMessage);
begin
  case Msg.WParamLo of
    IDOK     : OnOK;
    IDCANCEL : OnCancel;
  end;
end;

procedure TDialog.WMCtlColorBtn(var Msg: TMessage);
begin

end;

procedure TDialog.WMCtlColorDlg(var Msg: TMessage);
begin

end;

procedure TDialog.WMCtlColorEdit(var Msg: TMessage);
begin

end;

procedure TDialog.WMCtlColorListBox(var Msg: TMessage);
begin

end;

procedure TDialog.WMCtlColorStatic(var Msg: TMessage);
begin

end;

procedure TDialog.WMInitDialog(var Msg: TMessage);
begin
  Msg.Result := LRESULT( OnInitDialog );
end;

procedure TDialog.WndProc(var Msg: TMessage);
begin
  case Msg.Msg of
    WM_COMMAND:     WMCommand( Msg );
    WM_INITDIALOG:  WMInitDialog( Msg );
    WM_CTLCOLORBTN: WMCtlColorBtn( Msg );
  end;
end;

{ TPropertiesDialog }

constructor TPropertiesDialog.Create(Parent: HWND);
begin
  inherited Create( Parent, 101 );
end;

destructor TPropertiesDialog.Destroy;
begin
  inherited Destroy;
end;

function TPropertiesDialog.OnInitDialog: BOOL;
begin
  Result := inherited OnInitDialog;
end;

procedure TPropertiesDialog.WMInitDialog(var Msg: TMessage);
begin
  inherited WMInitDialog( Msg );
end;

procedure TPropertiesDialog.WndProc(var Msg: TMessage);
begin
  inherited WndProc( Msg );
end;

const
  CM_MOUSEENTER    = WM_APP + 1;
  CM_MOUSELEAVE    = WM_APP + 2;
  CM_CURSORCHANGED = WM_APP + 3;

var
  LastControl: TControl = nil;
  hWndLast: HWND = 0;
  
function DefControlProc( hDlg: HWND; uMsg: UINT; wParam, lParam: LongInt ): LRESULT; stdcall;
var
  Control: TControl;
  Msg: TMessage;
begin

  Result := 0;

  Msg.Msg := uMsg;
  Msg.WParam := wParam;
  Msg.LParam := lParam;
  Msg.Result := Integer( Result );

  //Control := TControl( GetWindowLong( hDlg, GWL_USERDATA ) );
  Control := TControl( GetProp( hDlg, 'Self_Control' ) );

  if Assigned( Control ) then
  begin
    if uMsg = WM_NCDESTROY then
    begin
      RemoveProp( hDlg, 'Self_Control' );
      SetWindowLong( hDlg, GWL_WNDPROC, LongInt(@Control.ControlProc) );
      Control.WndProc( Msg );
      Control.Free;
    end
    else Control.WndProc( Msg );
  end;

  Result := Msg.Result;

end;

{ TControl }

procedure TControl.CMMouseEnter(var Msg: TMessage);
{
var
  Style: DWORD;
}
begin
  //SetWindowText( Parent, PChar(Format('Enter %d', [FControlId])) );
  //TextColor := RGB(0, 0, 255);
  {
  Style := ExStyle;
  if Style and WS_EX_STATICEDGE = 0 then begin
    Style := Style or WS_EX_STATICEDGE;
    ExStyle := Style;
  end;
  }
  {
  Style := GetWindowLong( FHandle, GWL_EXSTYLE );
  if Style and WS_EX_STATICEDGE = 0 then begin
    Style := Style or WS_EX_STATICEDGE;
    SetWindowLong( FHandle, GWL_EXSTYLE, Style );
    if FHandle <> 0 then SetWindowPos( Handle, 0, 0, 0, 0, 0, //SWP_NOACTIVATE or
      SWP_NOZORDER or SWP_NOMOVE or SWP_NOSIZE or SWP_FRAMECHANGED );
  end;
  }
  {
  Style := GetWindowLong( FHandle, GWL_STYLE );
  if Style and WS_BORDER = 0 then begin
    Style := Style or WS_BORDER;
    SetWindowLong( FHandle, GWL_STYLE, Style );
    if FHandle <> 0 then SetWindowPos( Handle, 0, 0, 0, 0, 0, //SWP_NOACTIVATE or
      SWP_NOZORDER or SWP_NOMOVE or SWP_NOSIZE or SWP_FRAMECHANGED );//or SWP_DRAWFRAME );//
  end;
  }
  if Assigned( FMouseEnter ) then FMouseEnter(Self);
end;

procedure TControl.CMMouseLeave(var Msg: TMessage);
{
var
  Style: DWORD;
}
begin
  //SetWindowText( Parent, PChar(Format('Leave %d', [FControlId])) );
  //TextColor := RGB(0, 0, 0);
  {
  Style := ExStyle;
  if Style and WS_EX_STATICEDGE <> 0 then begin
    Style := Style and (not WS_EX_STATICEDGE);
    ExStyle := Style;
  end;
  }
  {
  Style := GetWindowLong( FHandle, GWL_EXSTYLE );
  if Style and WS_EX_STATICEDGE <> 0 then begin
    Style := Style and (not WS_EX_STATICEDGE);
    SetWindowLong( FHandle, GWL_EXSTYLE, Style );
    if FHandle <> 0 then SetWindowPos(Handle, 0, 0, 0, 0, 0, //SWP_NOACTIVATE or
      SWP_NOZORDER or SWP_NOMOVE or SWP_NOSIZE or SWP_FRAMECHANGED);
  end;
  }
  {
  Style := GetWindowLong( FHandle, GWL_STYLE );
  if Style and WS_BORDER <> 0 then begin
    Style := Style and (not WS_BORDER);
    SetWindowLong( FHandle, GWL_STYLE, Style );
    if FHandle <> 0 then SetWindowPos(Handle, 0, 0, 0, 0, 0, //SWP_NOACTIVATE or
      SWP_NOZORDER or SWP_NOMOVE or SWP_NOSIZE or SWP_FRAMECHANGED );//or SWP_DRAWFRAME );
  end;
  }
  if Assigned( FMouseLeave ) then FMouseLeave(Self);
end;

constructor TControl.Create(Parent: HWND);
begin
  inherited Create;
  FParent := Parent;
end;

procedure TControl.DefaultProc(var Msg: TMessage);
begin
  with Msg do
    Result := CallWindowProc( ControlProc, Handle, Msg, WParam, LParam );
end;

destructor TControl.Destroy;
begin
  if FPopupMenu <> 0 then DestroyMenu( FPopupMenu );
  if FBrush <> 0 then DeleteObject( FBrush );
  inherited Destroy;
end;

function TControl.GetExStyle: DWORD;
begin
  Result := FExStyle;
end;

function TControl.Perform(Msg: UINT; WParam, LParam: Integer): LongInt;
var
  Message: TMessage;
begin
  Message.Msg := Msg;
  Message.WParam := WParam;
  Message.LParam := LParam;
  Message.Result := 0;
  if Self <> nil then WndProc( Message );
  Result := Message.Result;
end;

procedure TControl.SetColor(const Value: DWORD);
begin
  //if FColor <> Value then
  begin
    FColor := Value;
    if FBrush <> 0 then
      DeleteObject( FBrush );
    FBrush := CreateSolidBrush( FColor );
    if IsWindow(FHandle) then InvalidateRect( FHandle, nil, True );
  end;
end;

procedure TControl.SetCursor(const Value: HCURSOR);
begin
  if FCursor <> Value then begin
    FCursor := Value;
    Perform( CM_CURSORCHANGED, 0, 0 );
  end;
end;

procedure TControl.SetExStyle(const Value: DWORD);
begin
  if FExStyle <> Value then
  begin
    FExStyle := Value;
    SetWindowLong( FHandle, GWL_EXSTYLE, FExStyle );
    if FHandle <> 0 then SetWindowPos(Handle, 0, 0, 0, 0, 0, //SWP_NOACTIVATE or
      SWP_NOZORDER or SWP_NOMOVE or SWP_NOSIZE or SWP_FRAMECHANGED);
  end;
end;

procedure TControl.SetFont(const Value: HFONT);
begin
  if FFont <> Value then begin
    FFont := Value;
    Perform( WM_SETFONT, FFont, Integer(True) );
  end;
end;

procedure TControl.SetHandle(const Value: HWND);
begin
  FHandle := Value;
  FControlId := GetDlgCtrlID( FHandle );
  FCursor := GetClassLong( FHandle, GCL_HCURSOR );
  FExStyle := GetWindowLong( FHandle, GWL_EXSTYLE );
end;

procedure TControl.SetTextColor(const Value: DWORD);
begin
  if FTextColor <> Value then begin
    FTextColor := Value;
    if IsWindow(FHandle) then InvalidateRect( FHandle, nil, True );
  end;
end;

procedure TControl.WMContextMenu(var Msg: TMessage);
begin
  if FPopupMenu <> 0 then begin
    TrackPopupMenu( FPopupMenu, TPM_LEFTALIGN or TPM_LEFTBUTTON,
      LoWord( Msg.LParam ), HiWord( Msg.LParam ), 0, Parent, nil );
  end;
end;

procedure TControl.WMMouseMove(var Msg: TMessage);
begin
  {
  if LastControl <> Self then begin
    if Assigned( LastControl ) then
      LastControl.Perform( CM_MOUSELEAVE, Msg.WParam, Msg.LParam );
    if Assigned( Self ) then
      Self.Perform( CM_MOUSEENTER, Msg.WParam, Msg.LParam );
    LastControl := Self;
  end;
  }
  if hWndLast <> FHandle then begin
    if IsWindow( hWndLast ) then
      SendMessage( hWndLast, CM_MOUSELEAVE, Msg.WParam, Msg.LParam );
    if IsWindow( FHandle ) then
      Self.Perform( CM_MOUSEENTER, Msg.WParam, Msg.LParam );
    hWndLast := FHandle;
  end;

end;

procedure TControl.WMMove(var Msg: TMessage);
begin
  FLeft := LoWord(Msg.LParam);
  FTop := HiWord(Msg.LParam);
end;

procedure TControl.WMSetCursor(var Msg: TMessage);
begin
  with Msg do begin
    case LoWord(LParam) of
      HTCLIENT:
        if FCursor <> 0 then begin
          Windows.SetCursor(FCursor);
          Result := 1;
          Exit;
        end;
      //HTERROR:;
    end;
  end;
end;

procedure TControl.WMSetFont(var Msg: TMessage);
begin
  FFont := Msg.WParam;
end;

procedure TControl.WMWindowPosChanged(var Msg: TMessage);
var
  WindowPos: PWINDOWPOS;
begin
  WindowPos := PWINDOWPOS(Msg.LParam);
  with WindowPos^ do begin
    FLeft := x;
    FTop := y;
  end;
end;

procedure TControl.WndProc(var Msg: TMessage);
begin
  {
  case Msg.Msg of
    WM_SETFONT: WMSetFont( Msg );
    WM_CONTEXTMENU: WMContextMenu( Msg );
    WM_MOUSEMOVE: WMMouseMove( Msg );
    WM_WINDOWPOSCHANGED: WMWindowPosChanged( Msg );
    WM_MOVE: WMMove( Msg );
  else
  end;
  }
  case Msg.Msg of
    WM_MOUSEMOVE: WMMouseMove( Msg );
    WM_SETCURSOR:
      begin
        WMSetCursor( Msg );
        Exit;
      end;
    CM_MOUSEENTER:
      begin
        CMMouseEnter( Msg );
        Exit;
      end;
    CM_MOUSELEAVE:
      begin
        CMMouseLeave( Msg );
        Exit;
      end;
    WM_CONTEXTMENU:
      begin
        if FPopupMenu <> 0 then begin
          WMContextMenu( Msg );
          Exit;
        end;
      end;
  end;
  DefaultProc( Msg );
end;

{ TAppDialog }

procedure TAppDialog.CMMouseEnter(var Msg: TMessage);
begin

end;

procedure TAppDialog.CMMouseLeave(var Msg: TMessage);
begin

end;

constructor TAppDialog.Create(Id: UINT);
var
  ncm: NONCLIENTMETRICS;
  lf: LOGFONT;
  dc: HDC;
begin
  inherited Create( HWND_DESKTOP, Id );
  dc := GetDC( HWND_DESKTOP );
  ncm.cbSize := SizeOf( ncm );
  SystemParametersInfo( SPI_GETNONCLIENTMETRICS, 0, @ncm, 0 );
  System.Move( ncm.lfMessageFont, lf, SizeOf( LOGFONT ) );
  lf.lfWeight := FW_BOLD;
  lf.lfUnderline := Byte(True);
  lstrcpy( lf.lfFaceName, 'Tahoma' );
  lf.lfCharSet := TURKISH_CHARSET;
  lf.lfHeight := - MulDiv( GetDeviceCaps( dc, LOGPIXELSY ), 8, 72 );
  FFont := CreateFontIndirect( lf );
  ReleaseDC( HWND_DESKTOP, dc );
end;

destructor TAppDialog.Destroy;
begin
  DeleteObject( FFont );
  inherited Destroy;
end;

function EnumChildProc(Wnd: HWND; lParam: Integer): BOOL; stdcall;
var
  Parent: TAppDialog;
  Control: TControl;
  wc: WNDCLASS;
  ControlClass: TControlClass;
  ClassName: array[0..MAX_PATH-1] of Char;
begin
  FillChar( ClassName, Sizeof(ClassName), 0 );
  GetClassName( Wnd, ClassName, SizeOf(ClassName) );
  if lstrcmpi( ClassName, 'EDIT' ) = 0 then
  begin
    GetClassInfo( SysInit.HInstance, ClassName, wc );
    ControlClass := TEdit;
  end
  else if lstrcmpi( ClassName, 'BUTTON' ) = 0 then
  begin
    GetClassInfo( SysInit.HInstance, ClassName, wc );
    ControlClass := TButton;
  end
  else if lstrcmpi( ClassName, 'LISTBOX' ) = 0 then
  begin
    GetClassInfo( SysInit.HInstance, ClassName, wc );
    ControlClass := TListBox;
  end
  else
  if lstrcmpi( ClassName, 'STATIC' ) = 0 then
  begin
    GetClassInfo( SysInit.HInstance, ClassName, wc );
    ControlClass := TStaticText;
  end
  else ControlClass := TControl;
  Parent := TAppDialog(lParam);
  Control := ControlClass.Create( Parent.Handle );
  SetWindowLong( Wnd, GWL_USERDATA, LongInt( Control ) );
  SetProp( Wnd, 'Self_Control', THandle(Control) );
  with Control do begin
    Handle := Wnd;
    ControlProc := Pointer(GetWindowLong( Wnd, GWL_WNDPROC ));
    {
    if ControlId = 1000 then
      Color := GetSysColor( COLOR_INFOBK )
    else if ControlId = 1008 then
      Color := RGB( 128, 0, 0 );
    }
    {
    PopupMenu := GetSubMenu( LoadMenu( SysInit.HInstance,
      MakeIntResource( 103 ) ), 0 );
    }
  end;
  SetWindowLong( Wnd, GWL_WNDPROC, LongInt(@DefControlProc) );
  Parent.IncrementCount;
  Result := True;
end;

procedure TAppDialog.DoMouseEnter(Sender: TControl);
begin
  Sender.TextColor := RGB( 0, 0, 255 );
end;

procedure TAppDialog.DoMouseLeave(Sender: TControl);
begin
  Sender.TextColor := RGB( 255, 0, 0 );
end;

function TAppDialog.GetControl(Item: Integer): TControl;
var
  Window: HWND;
begin
  Window := Windows.GetDlgItem( Handle, Item );
  if Window <> 0 then begin
    Result := TControl(GetWindowLong( Window, GWL_USERDATA ));
  end else Result := nil;
end;

function TAppDialog.GetDlgItem(Id: Integer): TControl;
var
  Window: HWND;
begin
  Window := Windows.GetDlgItem( Handle, Id );
  if Window <> 0 then begin
    //Result := TControl(GetWindowLong( Window, GWL_USERDATA ));
    Result := TControl(GetProp( Window, 'Self_Control' ));
  end else Result := nil;
end;

function TAppDialog.IncrementCount: Integer;
begin
  Inc( FControlCount );
  Result := FControlCount;
end;

const
  IDC_FONT  = 1001;
  IDC_ADD   = 1002;
  IDC_DIR   = 1003;
  IDC_CLEAR = 1004;
  IDC_LIST  = 1008;

procedure TAppDialog.OnAdd(var Msg: TMessage);
var
  //List: TListBox;
  List: TControl;
  Dialog: TDialog;
begin
  List := GetDlgItem(IDC_LIST);
  if Assigned( List ) then begin
    if List is TListBox then
      (List as TListBox).Add( 'Sample' );//TimeToStr(Time) );
  end;
  Dialog := TDialog.Create(Handle, 101);
  try
    Dialog.ShowModal;
  finally
    Dialog.Free;
  end;
end;

procedure TAppDialog.OnClear(var Msg: TMessage);
var
  List: TControl;
begin
  List := GetDlgItem(IDC_LIST);
  if Assigned( List ) then begin
    if List is TListBox then
      (List as TListBox).Clear;
  end;
end;

procedure TAppDialog.OnDir(var Msg: TMessage);
var
  List: TControl;
begin
  List := GetDlgItem(IDC_LIST);
  if Assigned( List ) then begin
    if List is TListBox then
      with (List as TListBox) do begin
        Clear;
        GetDir( 'C:\Windows\*.*', DDL_ARCHIVE );
      end;
  end;
end;

procedure TAppDialog.OnFont(var Msg: TMessage);
const
  IDC_EDIT = 1000;
var
  OkBtn, Static, Edit: TControl;
begin
  OkBtn := Controls[IDOK];
  if Assigned( OkBtn ) then begin
    OkBtn.Font := FFont;
  end;
  Static := Controls[1010];
  if Assigned( Static ) then begin
    Static.Font := FFont;
    Static.Color := GetSysColor( COLOR_BTNFACE );//RGB(128, 0, 0);
    Static.TextColor := RGB(255, 0, 0);//GetSysColor( COLOR_BTNTEXT );//
    Static.Cursor := LoadCursor( 0, IDC_HAND );
  end;
  Edit := GetDlgItem(IDC_EDIT);
  if Assigned( Edit ) then begin
    Edit.Font := FFont;
    Edit.Color := RGB(0, 128, 0);
    Edit.TextColor := GetSysColor( COLOR_WINDOW );
  end;
end;

function TAppDialog.OnInitDialog: BOOL;
var
  Edit: TControl;
  UrlText: TControl;
begin

  Result := inherited OnInitDialog;
  SetProcControls;

  Edit := GetDlgItem(2);
  if Assigned( Edit ) then begin
    Edit.Font := FFont;
    Edit.Color := RGB(255, 128, 0);
    Edit.TextColor := GetSysColor( COLOR_WINDOW );
    Edit.Cursor := LoadCursor( 0, IDC_HAND );
    Edit.PopupMenu := GetSubMenu( LoadMenu( SysInit.HInstance,
      MakeIntResource( 103 ) ), 0 );
  end;

  Edit := GetDlgItem(1001);
  if Assigned( Edit ) then begin
    Edit.Cursor := LoadCursor( 0, IDC_HAND );
  end;

  Edit := GetDlgItem(IDC_LIST);
  if Assigned( Edit ) then begin
    Edit.PopupMenu := GetSubMenu( LoadMenu( SysInit.HInstance,
      MakeIntResource( 103 ) ), 1 );
  end;

  UrlText := GetDlgItem(1010);
  if Assigned( UrlText ) then begin
    UrlText.MouseEnter := DoMouseEnter;
    UrlText.MouseLeave := DoMouseLeave;
    UrlText.Cursor := LoadCursor( 0, IDC_HAND );
  end;

end;

procedure TAppDialog.SetProcControls;
begin
  EnumChildWindows( Handle, @EnumChildProc, Integer(Self) );
end;

procedure TAppDialog.WMCommand(var Msg: TMessage);
begin
  inherited WMCommand( Msg );
  case Msg.WParamLo of
    IDC_FONT: OnFont( Msg );
    IDC_ADD : OnAdd( Msg );
    IDC_DIR : OnDir( Msg );
    IDC_CLEAR : OnClear( Msg );
  end;
end;

procedure TAppDialog.WMCtlColorBtn(var Msg: TMessage);
var
  Button: TControl;
begin
  Button := Controls[GetDlgCtrlID( Msg.LParam )];
  if Assigned( Button ) then begin
    Msg.Result := Button.Brush;
  end;
end;

procedure TAppDialog.WMCtlColorDlg(var Msg: TMessage);
begin
  Msg.Result := 0; //GetStockObject( GRAY_BRUSH );
end;

procedure TAppDialog.WMCtlColorEdit(var Msg: TMessage);
var
  Edit: TControl;
begin
  Edit := Controls[GetDlgCtrlID( Msg.LParam )];
  if Assigned( Edit ) then begin
    SetBkMode( Msg.WParam, TRANSPARENT );
    SetTextColor( Msg.WParam, Edit.TextColor );
    Msg.Result := Edit.Brush;
  end;
end;

procedure TAppDialog.WMCtlColorListBox(var Msg: TMessage);
var
  ListBox: TControl;
begin
  ListBox := Controls[GetDlgCtrlID( Msg.LParam )];
  if Assigned( ListBox ) then begin
    Msg.Result := ListBox.Brush;
  end;
end;

procedure TAppDialog.WMCtlColorStatic(var Msg: TMessage);
var
  Static: TControl;
begin
  Static := Controls[GetDlgCtrlID( Msg.LParam )];
  if Assigned( Static ) then begin
    SetBkMode( Msg.WParam, TRANSPARENT );
    SetTextColor( Msg.WParam, Static.TextColor );
    Msg.Result := Static.Brush;
  end;
end;

procedure TAppDialog.WMMouseMove(var Msg: TMessage);
begin
  if hWndLast <> FHandle then begin
    if IsWindow( hWndLast ) then
      SendMessage( hWndLast, CM_MOUSELEAVE, Msg.WParam, Msg.LParam );
    if IsWindow( FHandle ) then
      SendMessage( FHandle, CM_MOUSEENTER, Msg.WParam, Msg.LParam );
    hWndLast := FHandle;
  end;
end;

procedure TAppDialog.WndProc(var Msg: TMessage);
begin
  case Msg.Msg of
    WM_CTLCOLORBTN: WMCtlColorBtn( Msg );
    WM_CTLCOLORDLG: WMCtlColorDlg( Msg );
    WM_CTLCOLOREDIT: WMCtlColorEdit( Msg );
    WM_CTLCOLORLISTBOX: WMCtlColorListBox( Msg );
    WM_CTLCOLORSTATIC: WMCtlColorStatic( Msg );
    WM_MOUSEMOVE: WMMouseMove( Msg );
  else
    inherited WndProc( Msg );
  end;
end;

{ TButton }

constructor TButton.Create(Parent: HWND);
begin
  inherited Create(Parent);
end;

destructor TButton.Destroy;
begin
  inherited Destroy;
end;

{ TEdit }

constructor TEdit.Create(Parent: HWND);
begin
  inherited Create(Parent);
end;

destructor TEdit.Destroy;
begin
  inherited Destroy;
end;

{ TStaticText }

constructor TStaticText.Create(Parent: HWND);
begin
  inherited Create(Parent);
  Color := GetSysColor( COLOR_BTNFACE );
end;

destructor TStaticText.Destroy;
begin
  inherited Destroy;
end;

{ TListBox }

function TListBox.Add(const S: string): LongInt;
begin
  Result := Perform( LB_ADDSTRING, 0, LongInt(PChar(S)) );
end;

function TListBox.Clear: LongInt;
begin
  Result := Perform( LB_RESETCONTENT, 0, 0 );
end;

constructor TListBox.Create(Parent: HWND);
begin
  inherited Create(Parent);
end;

destructor TListBox.Destroy;
begin
  inherited Destroy;
end;

function TListBox.GetDir(const FileSpec: string; Attrs: UINT): LongInt;
begin
  Result := Perform( LB_DIR, Attrs, LongInt(PChar(FileSpec)) );
end;

{ TURLText }

procedure TURLText.CMMouseEnter(var Msg: TMessage);
begin
  TextColor := RGB(0, 0, 255);
end;

procedure TURLText.CMMouseLeave(var Msg: TMessage);
begin
  TextColor := RGB(255, 0, 0);
end;

constructor TURLText.Create(Parent: HWND);
begin
  inherited Create(Parent);
end;

destructor TURLText.Destroy;
begin
  inherited Destroy;
end;

{ TApplication }

constructor TApplication.Create;
begin
  inherited Create;
  FMainWnd := 0;
end;

destructor TApplication.Destroy;
begin
  inherited Destroy;
end;

function TApplication.InitInstance: Boolean;
begin
  Result := True;
end;

procedure TApplication.Run;
begin
  if InitInstance then begin

  end;
end;

end.
