(*
 *  FDI.H -- File Decompression Interface
 *
 *  Copyright (C) Microsoft Corporation 1993-1997
 *  All Rights Reserved.
 *)

{********************************************************************}
{                                                                    }
{       Delphi 5 conversion by Nasir Senturk <shenturk@yahoo.com>    }
{       Copyright (c) 2000-2005, Nasir Senturk, all rights reserved  }
{                                                                    }
{********************************************************************}

unit Cabinets;

interface

uses Windows;
(*
The CFHEADER structure provides information about this cabinet file.

struct CFHEADER
{
    u1  signature[4];	/* cabinet file signature */
    u4  reserved1;	/* reserved */
    u4  cbCabinet;	/* size of this cabinet file in bytes */
    u4  reserved2;	/* reserved */
    u4  coffFiles;	/* offset of the first CFFILE entry */
    u4  reserved3;	/* reserved */
    u1  versionMinor;	/* cabinet file format version, minor */
    u1  versionMajor;	/* cabinet file format version, major */
    u2  cFolders;	/* number of CFFOLDER entries in this cabinet */
    u2  cFiles;	/* number of CFFILE entries in this cabinet */
    u2  flags;	/* cabinet file option indicators */
    u2  setID;	/* must be the same for all cabinets in a set */
    u2  iCabinet;	/* number of this cabinet file in a set */
    u2  cbCFHeader; 	/* (optional) size of per-cabinet reserved area */
    u1  cbCFFolder; 	/* (optional) size of per-folder reserved area */
    u1  cbCFData; 	/* (optional) size of per-datablock reserved area */
    u1  abReserve[];	/* (optional) per-cabinet reserved area */
    u1  szCabinetPrev[];	/* (optional) name of previous cabinet file */
    u1  szDiskPrev[];	/* (optional) name of previous disk */
    u1  szCabinetNext[];	/* (optional) name of next cabinet file */
    u1  szDiskNext[];	/* (optional) name of next disk */
};

*)
type
  u1 = Char;
  u2 = Word;
  u4 = LongWord;

  CFOPTIONAL = packed record
    cbCFHeader: u2;
    cbCFFolder: u1;
    cbCFData: u1;
    abReserve: array[0..0] of u1;
    szCabinetPrev: array[0..0] of u1;
    szDiskPrev: array[0..0] of u1;
    szCabinetNext: array[0..0] of u1;
    szDiskNext: array[0..0] of u1;
  end;

  CFHEADER = packed record
    signature: array[0..3] of u1;
    reserved1: u4;
    cbCabinet: u4;
    reserved2: u4;
    coffFiles: u4;
    reserved3: u4;
    versionMinor: u1;
    versionMajor: u1;
    cFolders: u2;
    cFiles: u2;
    flags: u2;
    setID: u2;
    iCabinet: u2;
  end;

(*
struct CFFOLDER
{
    u4  coffCabStart;	/* offset of the first CFDATA block in this folder */
    u2  cCFData;	/* number of CFDATA blocks in this folder */
    u2  typeCompress;	/* compression type indicator */
    u1  abReserve[];	/* (optional) per-folder reserved area */
};

*)
  CFFOLDER = packed record
    coffCabStart: u4;
    cCFData: u2;
    typeCompress: u2;
    abReserve: array[0..0] of u1;
  end;

(*
struct CFFILE
{
    u4  cbFile;	/* uncompressed size of this file in bytes */
    u4  uoffFolderStart;	/* uncompressed offset of this file in the folder */
    u2  iFolder;	/* index into the CFFOLDER area */
    u2  date;	/* date stamp for this file */
    u2  time;	/* time stamp for this file */
    u2  attribs;	/* attribute flags for this file */
    u1  szName[];	/* name of this file */
};
*)
  CFFILE = packed record
    cbFile: u4;
    uoffFolderStart: u4;
    iFolder: u2;
    date: u2;
    time: u2;
    attribs: u2;
    szName: array[0..0] of u1;
  end;

(*
struct CFDATA
{
    u4  csum;	/* checksum of this CFDATA entry */
    u2  cbData;	/* number of compressed bytes in this block */
    u2  cbUncomp;	/* number of uncompressed bytes in this block */
    u1  abReserve[];	/* (optional) per-datablock reserved area */
    u1  ab[cbData];	/* compressed data bytes */
};
*)
  CFDATA = packed record
    csum: u4;
    cbData: u2;
    cbUncomp: u2;
    abReserve: array[0..0] of u1;
    ab: array[0..0] of u1;
  end;

type
  CHECKSUM = LongWord;
  UOFF     = LongWord;
  COFF     = LongWord;

(***    ERF - Error structure
 *
 *  This structure returns error information from FCI/FDI.  The caller should
 *  not modify this structure.
 *)

  ERF = record
    erfOper: Integer;  // FCI/FDI error code -- see FDIERROR_XXX
                       //  and FCIERR_XXX equates for details.
    erfType: Integer;  // Optional error value filled in by FCI/FDI.
                       // For FCI, this is usually the C run-time
                       // *errno* value.
    fError: BOOL;      // TRUE => error present
  end;
  PERF = ^ERF;

const
  CB_MAX_CHUNK        =    32768;
  CB_MAX_DISK         = $7ffffff;
  CB_MAX_FILENAME     =      256;
  CB_MAX_CABINET_NAME =      256;
  CB_MAX_CAB_PATH     =      256;
  CB_MAX_DISK_NAME    =      256;

type
  TCOMP = Word;

const
  tcompMASK_TYPE        =  $000F;  // Mask for compression type
  tcompTYPE_NONE        =  $0000;  // No compression
  tcompTYPE_MSZIP       =  $0001;  // MSZIP
  tcompTYPE_QUANTUM     =  $0002;  // Quantum
  tcompTYPE_LZX         =  $0003;  // LZX
  tcompBAD              =  $000F;  // Unspecified compression type

  tcompMASK_LZX_WINDOW  =  $1F00;  // Mask for LZX Compression Memory
  tcompLZX_WINDOW_LO    =  $0F00;  // Lowest LZX Memory (15)
  tcompLZX_WINDOW_HI    =  $1500;  // Highest LZX Memory (21)
  tcompSHIFT_LZX_WINDOW =      8;  // Amount to shift over to get int

  tcompMASK_QUANTUM_LEVEL = $00F0;  // Mask for Quantum Compression Level
  tcompQUANTUM_LEVEL_LO   = $0010;  // Lowest Quantum Level (1)
  tcompQUANTUM_LEVEL_HI   = $0070;  // Highest Quantum Level (7)
  tcompSHIFT_QUANTUM_LEVEL =    4;  // Amount to shift over to get int

  tcompMASK_QUANTUM_MEM  = $1F00;  // Mask for Quantum Compression Memory
  tcompQUANTUM_MEM_LO    = $0A00;  // Lowest Quantum Memory (10)
  tcompQUANTUM_MEM_HI    = $1500;  // Highest Quantum Memory (21)
  tcompSHIFT_QUANTUM_MEM =     8;  // Amount to shift over to get int

  tcompMASK_RESERVED     = $E000;  // Reserved bits (high 3 bits)

{ Macros }
function CompressionTypeFromTCOMP(tc: TCOMP): TCOMP;
function CompressionLevelFromTCOMP(tc: TCOMP): TCOMP;
function CompressionMemoryFromTCOMP(tc: TCOMP): TCOMP;
function TCOMPFromTypeLevelMemory(t,l,m: Word): TCOMP;
function LZXCompressionWindowFromTCOMP(tc: TCOMP): TCOMP;
function TCOMPFromLZXWindow(w: Word): TCOMP;

(***    FDIERROR - Error codes returned in erf.erfOper field
 *
 *  In general, FDI will only fail if one of the passed in memory or
 *  file I/O functions fails.  Other errors are pretty unlikely, and are
 *  caused by corrupted cabinet files, passing in a file which is not a
 *  cabinet file, or cabinet files out of order.
 *
 *  Description:    Summary of error.
 *  Cause:          List of possible causes of this error.
 *  Response:       How client might respond to this error, or avoid it in
 *                  the first place.
 *)
const
  FDIERROR_NONE = 0;
        // Description: No error
        // Cause:       Function was successfull.
        // Response:    Keep going!

  FDIERROR_CABINET_NOT_FOUND = 1;
        // Description: Cabinet not found
        // Cause:       Bad file name or path passed to FDICopy(), or returned
        //              to fdintNEXT_CABINET.
        // Response:    To prevent this error, validate the existence of the
        //              the cabinet *before* passing the path to FDI.

  FDIERROR_NOT_A_CABINET = 2;
        // Description: Cabinet file does not have the correct format
        // Cause:       File passed to to FDICopy(), or returned to
        //              fdintNEXT_CABINET, is too small to be a cabinet file,
        //              or does not have the cabinet signature in its first
        //              four bytes.
        // Response:    To prevent this error, call FDIIsCabinet() to check a
        //              cabinet before calling FDICopy() or returning the
        //              cabinet path to fdintNEXT_CABINET.

  FDIERROR_UNKNOWN_CABINET_VERSION = 3;
        // Description: Cabinet file has an unknown version number.
        // Cause:       File passed to to FDICopy(), or returned to
        //              fdintNEXT_CABINET, has what looks like a cabinet file
        //              header, but the version of the cabinet file format
        //              is not one understood by this version of FDI.  The
        //              erf.erfType field is filled in with the version number
        //              found in the cabinet file.
        // Response:    To prevent this error, call FDIIsCabinet() to check a
        //              cabinet before calling FDICopy() or returning the
        //              cabinet path to fdintNEXT_CABINET.

  FDIERROR_CORRUPT_CABINET = 4;
        // Description: Cabinet file is corrupt
        // Cause:       FDI returns this error any time it finds a problem
        //              with the logical format of a cabinet file, and any
        //              time one of the passed-in file I/O calls fails when
        //              operating on a cabinet (PFNOPEN, PFNSEEK, PFNREAD,
        //              or PFNCLOSE).  The client can distinguish these two
        //              cases based upon whether the last file I/O call
        //              failed or not.
        // Response:    Assuming this is not a real corruption problem in
        //              a cabinet file, the file I/O functions could attempt
        //              to do retries on failure (for example, if there is a
        //              temporary network connection problem).  If this does
        //              not work, and the file I/O call has to fail, then the
        //              FDI client will have to clean up and call the
        //              FDICopy() function again.

  FDIERROR_ALLOC_FAIL = 5;
        // Description: Could not allocate enough memory
        // Cause:       FDI tried to allocate memory with the PFNALLOC
        //              function, but it failed.
        // Response:    If possible, PFNALLOC should take whatever steps
        //              are possible to allocate the memory requested.  If
        //              memory is not immediately available, it might post a
        //              dialog asking the user to free memory, for example.
        //              Note that the bulk of FDI's memory allocations are
        //              made at FDICreate() time and when the first cabinet
        //              file is opened during FDICopy().

  FDIERROR_BAD_COMPR_TYPE = 6;
        // Description: Unknown compression type in a cabinet folder
        // Cause:       [Should never happen.]  A folder in a cabinet has an
        //              unknown compression type.  This is probably caused by
        //              a mismatch between the version of FCI.LIB used to
        //              create the cabinet and the FDI.LIB used to read the
        //              cabinet.
        // Response:    Abort.

  FDIERROR_MDI_FAIL = 7;
        // Description: Failure decompressing data from a cabinet file
        // Cause:       The decompressor found an error in the data coming
        //              from the file cabinet.  The cabinet file was corrupted.
        //              [11-Apr-1994 bens When checksuming is turned on, this
        //              error should never occur.]
        // Response:    Probably should abort; only other choice is to cleanup
        //              and call FDICopy() again, and hope there was some
        //              intermittent data error that will not reoccur.

  FDIERROR_TARGET_FILE = 8;
        // Description: Failure writing to target file
        // Cause:       FDI returns this error any time it gets an error back
        //              from one of the passed-in file I/O calls fails when
        //              writing to a file being extracted from a cabinet.
        // Response:    To avoid or minimize this error, the file I/O functions
        //              could attempt to avoid failing.  A common cause might
        //              be disk full -- in this case, the PFNWRITE function
        //              could have a check for free space, and put up a dialog
        //              asking the user to free some disk space.

  FDIERROR_RESERVE_MISMATCH = 9;
        // Description: Cabinets in a set do not have the same RESERVE sizes
        // Cause:       [Should never happen]. FDI requires that the sizes of
        //              the per-cabinet, per-folder, and per-data block
        //              RESERVE sections be consistent across all the cabinets
        //              in a set.
        // Response:    Abort.

  FDIERROR_WRONG_CABINET = 10;
        // Description: Cabinet returned on fdintNEXT_CABINET is incorrect
        // Cause:       NOTE: THIS ERROR IS NEVER RETURNED BY FDICopy()!
        //              Rather, FDICopy() keeps calling the fdintNEXT_CABINET
        //              callback until either the correct cabinet is specified,
        //              or you return ABORT.
        //              When FDICopy() is extracting a file that crosses a
        //              cabinet boundary, it calls fdintNEXT_CABINET to ask
        //              for the path to the next cabinet.  Not being very
        //              trusting, FDI then checks to make sure that the
        //              correct continuation cabinet was supplied!  It does
        //              this by checking the "setID" and "iCabinet" fields
        //              in the cabinet.  When MAKECAB.EXE creates a set of
        //              cabinets, it constructs the "setID" using the sum
        //              of the bytes of all the destination file names in
        //              the cabinet set.  FDI makes sure that the 16-bit
        //              setID of the continuation cabinet matches the
        //              cabinet file just processed.  FDI then checks that
        //              the cabinet number (iCabinet) is one more than the
        //              cabinet number for the cabinet just processed.
        // Response:    You need code in your fdintNEXT_CABINET (see below)
        //              handler to do retries if you get recalled with this
        //              error.  See the sample code (EXTRACT.C) to see how
        //              this should be handled.

  FDIERROR_USER_ABORT = 11;
        // Description: FDI aborted.
        // Cause:       An FDI callback returnd -1 (usually).
        // Response:    Up to client.

type
  FDIERROR = Integer;

const
(*
 * FAT file attribute flag used by FCI/FDI to indicate that
 * the filename in the CAB is a UTF string
 *)
  _A_NAME_IS_UTF  = $80;

(*
 * FAT file attribute flag used by FCI/FDI to indicate that
 * the file should be executed after extraction
 *)
  _A_EXEC         = $40;

(***    HFDI - Handle to an FDI context
 *
 *  FDICreate() creates this, and it must be passed to all other FDI
 *  functions.
 *)
type
  HFDI = Pointer;

(***    FDICABINETINFO - Information about a cabinet
 *
 *)
  FDICABINETINFO = record
    cbCabinet  : LongInt;   // Total length of cabinet file
    cFolders   : Word;      // Count of folders in cabinet
    cFiles     : Word;      // Count of files in cabinet
    setID      : Word;      // Cabinet set ID
    iCabinet   : Word;      // Cabinet number in set (0 based)
    fReserve   : BOOL;      // TRUE => RESERVE present in cabinet
    hasprev    : BOOL;      // TRUE => Cabinet is chained prev
    hasnext    : BOOL;      // TRUE => Cabinet is chained next
  end;
  PFDICABINETINFO = ^FDICABINETINFO;

(***    FDIDECRYPTTYPE - PFNFDIDECRYPT command types
 *
 *)
const
  fdidtNEW_CABINET = 0;  // New cabinet
  fdidtNEW_FOLDER  = 1;  // New folder
  fdidtDECRYPT     = 2;  // Decrypt a data block

type
  FDIDECRYPTTYPE = Integer;

(***    FDIDECRYPT - Data for PFNFDIDECRYPT function
 *
 *)
  tagCabinet = record
    pHeaderReserve  : Pointer;
    cbHeaderReserve : Word;
    setID           : Word;
    iCabinet        : Integer;
  end;

  tagFolder = record
    pFolderReserve  : Pointer;
    cbFolderReserve : Word;
    iFolder         : Word;
  end;

  tagDecrypt = record
    pDataReserve    : Pointer;
    cbDataReserve   : Word;
    pbData          : Pointer;
    cbData          : Word;
    fSplit          : BOOL;
    cbPartial       : Word;
  end;

  FDIDECRYPT = record
    fdidt  : FDIDECRYPTTYPE;
    pvUser : Pointer;
    case Integer of
      0: (cabinet: tagCabinet);
      1: (folder: tagFolder);
      2: (decrypt: tagDecrypt);
  end;
  PFDIDECRYPT = ^FDIDECRYPT;
  
(***    FNALLOC - Memory Allocation
 *      FNFREE  - Memory Free
 *
 *  These are modeled after the C run-time routines malloc() and free()
 *  FDI expects error handling to be identical to these C run-time routines.
 *
 *  As long as you faithfully copy the semantics of malloc() and free(),
 *  you can supply any functions you like!
 *
 *  WARNING: You should never assume anything about the sequence of
 *           PFNALLOC and PFNFREE calls -- incremental releases of
 *           FDI may have radically different numbers of
 *           PFNALLOC calls and allocation sizes!
 *)
//** Memory functions for FDI
type
  PFNALLOC = function (cb: Cardinal): Pointer; cdecl;
// typedef void HUGE * (FAR DIAMONDAPI *PFNALLOC)(ULONG cb); /* pfna */
// #define FNALLOC(fn) void HUGE * FAR DIAMONDAPI fn(ULONG cb)

  PFNFREE = procedure (pv: Pointer); cdecl;
// typedef void (FAR DIAMONDAPI *PFNFREE)(void HUGE *pv); /* pfnf */
// #define FNFREE(fn) void FAR DIAMONDAPI fn(void HUGE *pv)

(***    PFNOPEN  - File I/O callbacks for FDI
 *      PFNREAD
 *      PFNWRITE
 *      PFNCLOSE
 *      PFNSEEK
 *
 *  These are modeled after the C run-time routines _open, _read,
 *  _write, _close, and _lseek.  The values for the PFNOPEN oflag
 *  and pmode calls are those defined for _open.  FDI expects error
 *  handling to be identical to these C run-time routines.
 *
 *  As long as you faithfully copy these aspects, you can supply
 *  any functions you like!
 *
 *  WARNING: You should never assume you know what file is being
 *           opened at any one point in time!  FDI will usually
 *           stick to opening cabinet files, but it is possible
 *           that in a future implementation it may open temporary
 *           files or open cabinet files in a different order.
 *
 *  Notes for Memory Mapped File fans:
 *      You can write wrapper routines to allow FDI to work on memory
 *      mapped files.  You'll have to create your own "handle" type so that
 *      you can store the base memory address of the file and the current
 *      seek position, and then you'll allocate and fill in one of these
 *      structures and return a pointer to it in response to the PFNOPEN
 *      call and the fdintCOPY_FILE call.  Your PFNREAD and PFNWRITE
 *      functions will do memcopy(), and update the seek position in your
 *      "handle" structure.  PFNSEEK will just change the seek position
 *      in your "handle" structure.
 *)

//** File I/O functions for FDI
  PFNOPEN = function (pszFile: LPSTR; oflag: Integer; pmode: Integer): Integer; cdecl;
// typedef int  (FAR DIAMONDAPI *PFNOPEN) (char FAR *pszFile, int oflag, int pmode);
  PFNREAD = function (hf: Integer; pv: Pointer; cb: UINT): UINT; cdecl;
// typedef UINT (FAR DIAMONDAPI *PFNREAD) (int hf, void FAR *pv, UINT cb);
  PFNWRITE = function (hf: Integer; pv: Pointer; cb: UINT): UINT; cdecl;
// typedef UINT (FAR DIAMONDAPI *PFNWRITE)(int hf, void FAR *pv, UINT cb);
  PFNCLOSE = function (hf: Integer): Integer; cdecl;
// typedef int  (FAR DIAMONDAPI *PFNCLOSE)(int hf);
  PFNSEEK = function (hf: Integer; dist: LongInt; seektype: Integer): LongInt; cdecl;
// typedef long (FAR DIAMONDAPI *PFNSEEK) (int hf, long dist, int seektype);

(***    PFNFDIDECRYPT - FDI Decryption callback
 *
 *  If this function is passed on the FDICopy() call, then FDI calls it
 *  at various times to update the decryption state and to decrypt FCDATA
 *  blocks.
 *
 *  Common Entry Conditions:
 *      pfdid->fdidt  - Command type
 *      pfdid->pvUser - pvUser value from FDICopy() call
 *
 *  fdidtNEW_CABINET:   //** Notification of a new cabinet
 *      Entry:
 *        pfdid->cabinet.
 *          pHeaderReserve  - RESERVE section from CFHEADER
 *          cbHeaderReserve - Size of pHeaderReserve
 *          setID           - Cabinet set ID
 *          iCabinet        - Cabinet number in set (0 based)
 *      Exit-Success:
 *          returns anything but -1;
 *      Exit-Failure:
 *          returns -1; FDICopy() is aborted.
 *      Notes:
 *      (1) This call allows the decryption code to pick out any information
 *          from the cabinet header reserved area (placed there by DIACRYPT)
 *          needed to perform decryption.  If there is no such information,
 *          this call would presumably be ignored.
 *      (2) This call is made very soon after fdintCABINET_INFO.
 *
 *  fdidtNEW_FOLDER:    //** Notification of a new folder
 *      Entry:
 *        pfdid->folder.
 *          pFolderReserve  - RESERVE section from CFFOLDER
 *          cbFolderReserve - Size of pFolderReserve
 *          iFolder         - Folder number in cabinet (0 based)
 *      Exit-Success:
 *          returns anything but -1;
 *      Exit-Failure:
 *          returns -1; FDICopy() is aborted.
 *      Notes:
 *          This call allows the decryption code to pick out any information
 *          from the folder reserved area (placed there by DIACRYPT) needed
 *          to perform decryption.  If there is no such information, this
 *          call would presumably be ignored.
 *
 *  fdidtDECRYPT:       //** Decrypt a data buffer
 *      Entry:
 *        pfdid->folder.
 *          pDataReserve  - RESERVE section for this CFDATA block
 *          cbDataReserve - Size of pDataReserve
 *          pbData        - Data buffer
 *          cbData        - Size of data buffer
 *          fSplit        - TRUE if this is a split data block
 *          cbPartial     - 0 if this is not a split block, or the first
 *                              piece of a split block; Greater than 0 if
 *                              this is the second piece of a split block.
 *      Exit-Success:
 *          returns TRUE;
 *      Exit-Failure:
 *          returns FALSE; error during decrypt
 *          returns -1; FDICopy() is aborted.
 *      Notes:
 *          FCI will split CFDATA blocks across cabinet boundaries if
 *          necessary.  To provide maximum flexibility, FDI will call the
 *          fdidtDECRYPT function twice on such split blocks, once when
 *          the first portion is read, and again when the second portion
 *          is read.  And, of course, most data blocks will not be split.
 *          So, there are three cases:
 *
 *           1) fSplit == FALSE
 *              You have the entire data block, so decrypt it.
 *
 *           2) fSplit == TRUE, cbPartial == 0
 *              This is the first portion of a split data block, so cbData
 *              is the size of this portion.  You can either choose to decrypt
 *              this piece, or ignore this call and decrypt the full CFDATA
 *              block on the next (second) fdidtDECRYPT call.
 *
 *           3) fSplit == TRUE, cbPartial > 0
 *              This is the second portion of a split data block (indeed,
 *              cbPartial will have the same value as cbData did on the
 *              immediately preceeding fdidtDECRYPT call!).  If you decrypted
 *              the first portion on the first call, then you can decrypt the
 *              second portion now.  If you ignored the first call, then you
 *              can decrypt the entire buffer.
 *              NOTE: pbData points to the second portion of the split data
 *                    block in this case, *not* the entire data block.  If
 *                    you want to wait until the second piece to decrypt the
 *                    *entire* block, pbData-cbPartial is the address of the
 *                    start of the whole block, and cbData+cbPartial is its
 *                    size.
 *)
  PFNFDIDECRYPT = function (pfdid: PFDIDECRYPT): Integer; cdecl;
// typedef int (FAR DIAMONDAPI *PFNFDIDECRYPT)(PFDIDECRYPT pfdid); /* pfnfdid */

(***    FDINOTIFICATION - Notification structure for PFNFDINOTIFY
 *
 *  See the FDINOTIFICATIONTYPE definition for information on usage and
 *  meaning of these fields.
 *)
  FDINOTIFICATION = record
// long fields
    cb        : LongInt;
    psz1      : PChar;
    psz2      : PChar;
    psz3      : PChar;                // Points to a 256 character buffer
    pv        : Pointer;              // Value for client

// int fields
    hf        : Integer;

// short fields
    date      : Word;
    time      : Word;
    attribs   : Word;

    setID     : Word;                 // Cabinet set ID
    iCabinet  : Word;                 // Cabinet number (0-based)
    iFolder   : Word;                 // Folder number (0-based)

    fdie      : FDIERROR ;
  end;
  PFDINOTIFICATION = ^FDINOTIFICATION;

(***    FDINOTIFICATIONTYPE - FDICopy notification types
 *
 *  The notification function for FDICopy can be called with the following
 *  values for the fdint parameter.  In all cases, the pfdin->pv field is
 *  filled in with the value of the pvUser argument passed in to FDICopy().
 *
 *  A typical sequence of calls will be something like this:
 *      fdintCABINET_INFO     // Info about the cabinet
 *      fdintENUMERATE        // Starting enumeration
 *      fdintPARTIAL_FILE     // Only if this is not the first cabinet, and
 *                            // one or more files were continued from the
 *                            // previous cabinet.
 *      ...
 *      fdintPARTIAL_FILE
 *      fdintCOPY_FILE        // The first file that starts in this cabinet
 *      ...
 *      fdintCOPY_FILE        // Now let's assume you want this file...
 *      // PFNWRITE called multiple times to write to this file.
 *      fdintCLOSE_FILE_INFO  // File done, set date/time/attributes
 *
 *      fdintCOPY_FILE        // Now let's assume you want this file...
 *      // PFNWRITE called multiple times to write to this file.
 *      fdintNEXT_CABINET     // File was continued to next cabinet!
 *      fdintCABINET_INFO     // Info about the new cabinet
 *      // PFNWRITE called multiple times to write to this file.
 *      fdintCLOSE_FILE_INFO  // File done, set date/time/attributes
 *      ...
 *      fdintENUMERATE        // Ending enumeration
 *
 *  fdintCABINET_INFO:
 *        Called exactly once for each cabinet opened by FDICopy(), including
 *        continuation cabinets opened due to file(s) spanning cabinet
 *        boundaries. Primarily intended to permit EXTRACT.EXE to
 *        automatically select the next cabinet in a cabinet sequence even if
 *        not copying files that span cabinet boundaries.
 *      Entry:
 *          pfdin->psz1     = name of next cabinet
 *          pfdin->psz2     = name of next disk
 *          pfdin->psz3     = cabinet path name
 *          pfdin->setID    = cabinet set ID (a random 16-bit number)
 *          pfdin->iCabinet = Cabinet number within cabinet set (0-based)
 *      Exit-Success:
 *          Return anything but -1
 *      Exit-Failure:
 *          Returns -1 => Abort FDICopy() call
 *      Notes:
 *          This call is made *every* time a new cabinet is examined by
 *          FDICopy().  So if "foo2.cab" is examined because a file is
 *          continued from "foo1.cab", and then you call FDICopy() again
 *          on "foo2.cab", you will get *two* fdintCABINET_INFO calls all
 *          told.
 *
 *  fdintCOPY_FILE:
 *        Called for each file that *starts* in the current cabinet, giving
 *        the client the opportunity to request that the file be copied or
 *        skipped.
 *      Entry:
 *          pfdin->psz1    = file name in cabinet
 *          pfdin->cb      = uncompressed size of file
 *          pfdin->date    = file date
 *          pfdin->time    = file time
 *          pfdin->attribs = file attributes
 *          pfdin->iFolder = file's folder index
 *      Exit-Success:
 *          Return non-zero file handle for destination file; FDI writes
 *          data to this file use the PFNWRITE function supplied to FDICreate,
 *          and then calls fdintCLOSE_FILE_INFO to close the file and set
 *          the date, time, and attributes.  NOTE: This file handle returned
 *          must also be closeable by the PFNCLOSE function supplied to
 *          FDICreate, since if an error occurs while writing to this handle,
 *          FDI will use the PFNCLOSE function to close the file so that the
 *          client may delete it.
 *      Exit-Failure:
 *          Returns 0  => Skip file, do not copy
 *          Returns -1 => Abort FDICopy() call
 *
 *  fdintCLOSE_FILE_INFO:
 *        Called after all of the data has been written to a target file.
 *        This function must close the file and set the file date, time,
 *        and attributes.
 *      Entry:
 *          pfdin->psz1    = file name in cabinet
 *          pfdin->hf      = file handle
 *          pfdin->date    = file date
 *          pfdin->time    = file time
 *          pfdin->attribs = file attributes
 *          pfdin->iFolder = file's folder index
 *          pfdin->cb      = Run After Extract (0 - don't run, 1 Run)
 *      Exit-Success:
 *          Returns TRUE
 *      Exit-Failure:
 *          Returns FALSE, or -1 to abort;
 *
 *              IMPORTANT NOTE IMPORTANT:
 *                  pfdin->cb is overloaded to no longer be the size of
 *                  the file but to be a binary indicated run or not
 *
 *              IMPORTANT NOTE:
 *                  FDI assumes that the target file was closed, even if this
 *                  callback returns failure.  FDI will NOT attempt to use
 *                  the PFNCLOSE function supplied on FDICreate() to close
 *                  the file!
 *
 *  fdintPARTIAL_FILE:
 *        Called for files at the front of the cabinet that are CONTINUED
 *        from a previous cabinet.  This callback occurs only when FDICopy is
 *        started on second or subsequent cabinet in a series that has files
 *        continued from a previous cabinet.
 *      Entry:
 *          pfdin->psz1 = file name of file CONTINUED from a PREVIOUS cabinet
 *          pfdin->psz2 = name of cabinet where file starts
 *          pfdin->psz3 = name of disk where file starts
 *      Exit-Success:
 *          Return anything other than -1; enumeration continues
 *      Exit-Failure:
 *          Returns -1 => Abort FDICopy() call
 *
 *  fdintENUMERATE:
 *        Called once after a call to FDICopy() starts scanning a CAB's
 *        CFFILE entries, and again when there are no more CFFILE entries.
 *        If CAB spanning occurs, an additional call will occur after the
 *        first spanned file is completed.  If the pfdin->iFolder value is
 *        changed from zero, additional calls will occur next time it reaches
 *        zero.  If iFolder is changed to zero, FDICopy will terminate, as if
 *        there were no more CFFILE entries.  Primarily intended to allow an
 *        application with it's own file list to help FDI advance quickly to
 *        a CFFILE entry of interest.  Can also be used to allow an
 *        application to determine the cb values for each file in the CAB.
 *      Entry:
 *        pfdin->cb        = current CFFILE position
 *        pfdin->iFolder   = number of files remaining
 *        pfdin->setID     = current CAB's setID value
 *      Exit-Don't Care:
 *        Don't change anything.
 *        Return anything but -1.
 *      Exit-Forcing a skip:
 *        pfdin->cb        = desired CFFILE position
 *        pfdin->iFolder   = desired # of files remaining
 *        Return anything but -1.
 *      Exit-Stop:
 *        pfdin->iFolder    = set to 0
 *        Return anything but -1.
 *      Exit-Failure:
 *        Return -1 => Abort FDICopy call ("user aborted".)
 *      Notes:
 *        This call can be ignored by applications which want normal file
 *        searching.  The application can adjust the supplied values to
 *        force FDICopy() to continue it's search at another location, or
 *        to force FDICopy() to terminate the search, by setting iFolder to 0.
 *        (FDICopy() will report no error when terminated this way.)
 *        FDI has no means to verify the supplied cb or iFolder values.
 *        Arbitrary values are likely to cause undesirable results.  An
 *        application should cross-check pfdin->setID to be certain the
 *        external database is in sync with the CAB.  Reverse-skips are OK
 *        (but may be inefficient) unless fdintNEXT_CABINET has been called.
 *
 *  fdintNEXT_CABINET:
 *        This function is *only* called when fdintCOPY_FILE was told to copy
 *        a file in the current cabinet that is continued to a subsequent
 *        cabinet file.  It is important that the cabinet path name (psz3)
 *        be validated before returning!  This function should ensure that
 *        the cabinet exists and is readable before returning.  So, this
 *        is the function that should, for example, issue a disk change
 *        prompt and make sure the cabinet file exists.
 *
 *        When this function returns to FDI, FDI will check that the setID
 *        and iCabinet match the expected values for the next cabinet.
 *        If not, FDI will continue to call this function until the correct
 *        cabinet file is specified, or until this function returns -1 to
 *        abort the FDICopy() function.  pfdin->fdie is set to
 *        FDIERROR_WRONG_CABINET to indicate this case.
 *
 *        If you *haven't* ensured that the cabinet file is present and
 *        readable, or the cabinet file has been damaged, pfdin->fdie will
 *        receive other appropriate error codes:
 *
 *              FDIERROR_CABINET_NOT_FOUND
 *              FDIERROR_NOT_A_CABINET
 *              FDIERROR_UNKNOWN_CABINET_VERSION
 *              FDIERROR_CORRUPT_CABINET
 *              FDIERROR_BAD_COMPR_TYPE
 *              FDIERROR_RESERVE_MISMATCH
 *              FDIERROR_WRONG_CABINET
 *
 *      Entry:
 *          pfdin->psz1 = name of next cabinet where current file is continued
 *          pfdin->psz2 = name of next disk where current file is continued
 *          pfdin->psz3 = cabinet path name; FDI concatenates psz3 with psz1
 *                          to produce the fully-qualified path for the cabinet
 *                          file.  The 256-byte buffer pointed at by psz3 may
 *                          be modified, but psz1 may not!
 *          pfdin->fdie = FDIERROR_WRONG_CABINET if the previous call to
 *                        fdintNEXT_CABINET specified a cabinet file that
 *                        did not match the setID/iCabinet that was expected.
 *      Exit-Success:
 *          Return anything but -1
 *      Exit-Failure:
 *          Returns -1 => Abort FDICopy() call
 *      Notes:
 *          This call is almost always made when a target file is open and
 *          being written to, and the next cabinet is needed to get more
 *          data for the file.
 *)
const
  fdintCABINET_INFO    = 0;          // General information about cabinet
  fdintPARTIAL_FILE    = 1;          // First file in cabinet is continuation
  fdintCOPY_FILE       = 2;          // File to be copied
  fdintCLOSE_FILE_INFO = 3;          // close the file, set relevant info
  fdintNEXT_CABINET    = 4;          // File continued to next cabinet
  fdintENUMERATE       = 5;          // Enumeration status

type
  FDINOTIFICATIONTYPE = Integer;

  PFNFDINOTIFY = function (fdint: FDINOTIFICATIONTYPE; pfdin: PFDINOTIFICATION): Integer; cdecl;

(*** cpuType values for FDICreate()
 *
 *  (Ignored by 32-bit FDI.)
 *)
const
  cpuUNKNOWN       = -1;    { FDI does detection }
  cpu80286         = 0;     { '286 opcodes only }
  cpu80386         = 1;     { '386 opcodes used }

(***    FDICreate - Create an FDI context
 *
 *  Entry:
 *      pfnalloc
 *      pfnfree
 *      pfnopen
 *      pfnread
 *      pfnwrite
 *      pfnclose
 *      pfnlseek
 *      cpuType  - Select CPU type (auto-detect, 286, or 386+)
 *                 NOTE: For the 32-bit FDI.LIB, this parameter is ignored!
 *      perf
 *
 *  Exit-Success:
 *      Returns non-NULL FDI context handle.
 *
 *  Exit-Failure:
 *      Returns NULL; perf filled in with error code
 *
 *)
  function FDICreate(pfnalloc: PFNALLOC;
                     pfnfree: PFNFREE;
                     pfnopen: PFNOPEN;
                     pfnread: PFNREAD;
                     pfnwrite: PFNWRITE;
                     pfnclose: PFNCLOSE;
                     pfnseek: PFNSEEK;
                     cpuType: Integer;
                     perf: PERF): HFDI; cdecl;


(***    FDIIsCabinet - Determines if file is a cabinet, returns info if it is
 *
 *  Entry:
 *      hfdi   - Handle to FDI context (created by FDICreate())
 *      hf     - File handle suitable for PFNREAD/PFNSEEK, positioned
 *               at offset 0 in the file to test.
 *      pfdici - Buffer to receive info about cabinet if it is one.
 *
 *  Exit-Success:
 *      Returns TRUE; file is a cabinet, pfdici filled in.
 *
 *  Exit-Failure:
 *      Returns FALSE, file is not a cabinet;  If an error occurred,
 *          perf (passed on FDICreate call!) filled in with error.
 *)
  function FDIIsCabinet(hfdi: HFDI;
                        hf: Integer;
                        pfdici: PFDICABINETINFO): BOOL; cdecl;

(***    FDICopy - extracts files from a cabinet
 *
 *  Entry:
 *      hfdi        - handle to FDI context (created by FDICreate())
 *      pszCabinet  - main name of cabinet file
 *      pszCabPath  - Path to cabinet file(s)
 *      flags       - Flags to modify behavior
 *      pfnfdin     - Notification function
 *      pfnfdid     - Decryption function (pass NULL if not used)
 *      pvUser      - User specified value to pass to notification function
 *
 *  Exit-Success:
 *      Returns TRUE;
 *
 *  Exit-Failure:
 *      Returns FALSE, perf (passed on FDICreate call!) filled in with
 *          error.
 *
 *  Notes:
 *  (1) If FDICopy() fails while a target file is being written out, then
 *      FDI will use the PFNCLOSE function to close the file handle for that
 *      target file that was returned from the fdintCOPY_FILE notification.
 *      The client application is then free to delete the target file, since
 *      it will not be in a valid state (since there was an error while
 *      writing it out).
 *)
  function FDICopy(hfdi: HFDI;
                   pszCabinet: LPSTR;
                   pszCabPath: LPSTR;
                   flags: Integer;
                   pfnfdin: PFNFDINOTIFY;
                   pfnfdid: PFNFDIDECRYPT;
                   pvUser: Pointer): BOOL cdecl;

(***    FDIDestroy - Destroy an FDI context
 *
 *  Entry:
 *      hfdi - handle to FDI context (created by FDICreate())
 *
 *  Exit-Success:
 *      Returns TRUE;
 *
 *  Exit-Failure:
 *      Returns FALSE;
 *)
  function FDIDestroy(hfdi: HFDI): BOOL cdecl;

implementation

const
  Cabinetdll = 'cabinet.dll';

function CompressionTypeFromTCOMP(tc: TCOMP): TCOMP;
begin
  Result := ((tc) and tcompMASK_TYPE);
end;

function CompressionLevelFromTCOMP(tc: TCOMP): TCOMP;
begin
  Result := (((tc) and tcompMASK_QUANTUM_LEVEL) shr tcompSHIFT_QUANTUM_LEVEL);
end;

function CompressionMemoryFromTCOMP(tc: TCOMP): TCOMP;
begin
  Result := (((tc) and tcompMASK_QUANTUM_MEM) shr tcompSHIFT_QUANTUM_MEM);
end;

function TCOMPFromTypeLevelMemory(t,l,m: Word): TCOMP;
begin
  Result := (((m) shl tcompSHIFT_QUANTUM_MEM  ) or
            ((l) shl tcompSHIFT_QUANTUM_LEVEL) or
            ( t ));
end;

function LZXCompressionWindowFromTCOMP(tc: TCOMP): TCOMP;
begin
  Result := (((tc) and tcompMASK_LZX_WINDOW) shr tcompSHIFT_LZX_WINDOW);
end;

function TCOMPFromLZXWindow(w: Word): TCOMP;
begin
  Result := (((w) shl tcompSHIFT_LZX_WINDOW ) or
            ( tcompTYPE_LZX ));
end;

function FDICreate; external Cabinetdll name 'FDICreate';
function FDIIsCabinet; external Cabinetdll name 'FDIIsCabinet';
function FDICopy; external Cabinetdll name 'FDICopy';
function FDIDestroy; external Cabinetdll name 'FDIDestroy';

end.
